# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/execution/v1/deletion.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n qwak/execution/v1/deletion.proto\x12\x1fqwak.feature.store.execution.v1\"\x84\x01\n\x12\x46\x65\x61turesetDeletion\x12]\n\x19\x62\x61tch_featureset_deletion\x18\x01 \x01(\x0b\x32\x38.qwak.feature.store.execution.v1.BatchFeaturesetDeletionH\x00\x42\x0f\n\rdeletion_type\"\x19\n\x17\x42\x61tchFeaturesetDeletionB)\n%com.qwak.ai.features.execution.api.v1P\x01\x62\x06proto3')



_FEATURESETDELETION = DESCRIPTOR.message_types_by_name['FeaturesetDeletion']
_BATCHFEATURESETDELETION = DESCRIPTOR.message_types_by_name['BatchFeaturesetDeletion']
FeaturesetDeletion = _reflection.GeneratedProtocolMessageType('FeaturesetDeletion', (_message.Message,), {
  'DESCRIPTOR' : _FEATURESETDELETION,
  '__module__' : 'qwak.execution.v1.deletion_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.execution.v1.FeaturesetDeletion)
  })
_sym_db.RegisterMessage(FeaturesetDeletion)

BatchFeaturesetDeletion = _reflection.GeneratedProtocolMessageType('BatchFeaturesetDeletion', (_message.Message,), {
  'DESCRIPTOR' : _BATCHFEATURESETDELETION,
  '__module__' : 'qwak.execution.v1.deletion_pb2'
  # @@protoc_insertion_point(class_scope:qwak.feature.store.execution.v1.BatchFeaturesetDeletion)
  })
_sym_db.RegisterMessage(BatchFeaturesetDeletion)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n%com.qwak.ai.features.execution.api.v1P\001'
  _FEATURESETDELETION._serialized_start=70
  _FEATURESETDELETION._serialized_end=202
  _BATCHFEATURESETDELETION._serialized_start=204
  _BATCHFEATURESETDELETION._serialized_end=229
# @@protoc_insertion_point(module_scope)



from abc import ABC, abstractmethod
from dataclasses import dataclass
from enum import Enum
from typing import Type

from _qwak_proto.qwak.feature_store.sources.streaming_pb2 import (
    Authentication as ProtoAuthentication,
    Plain as ProtoPlain,
    Sasl as ProtoSasl,
    SaslMechanism as ProtoSaslMechanism,
    SecurityProtocol as ProtoSecurityProtocol,
    Ssl as ProtoSsl,
)
from qwak.clients.secret_service import SecretServiceClient
from qwak.exceptions import QwakException


class SaslMechanism(Enum):
    SCRAMSHA256 = ProtoSaslMechanism.SCRAMSHA256
    SCRAMSHA512 = ProtoSaslMechanism.SCRAMSHA512
    PLAIN = ProtoSaslMechanism.PLAIN


class SecurityProtocol(Enum):
    SASL_SSL = ProtoSecurityProtocol.SASL_SSL


@dataclass
class BaseAuthentication(ABC):
    @abstractmethod
    def _to_proto(self) -> ProtoAuthentication:
        pass

    @classmethod
    def _from_proto(
        cls,
        proto_authentication_method: ProtoAuthentication,
    ) -> Type["BaseAuthentication"]:
        proto_authentication_method = getattr(
            proto_authentication_method,
            proto_authentication_method.WhichOneof("type"),
        )
        if isinstance(proto_authentication_method, ProtoPlain):
            return PlainAuthentication._from_proto(proto_authentication_method)
        elif isinstance(proto_authentication_method, ProtoSsl):
            return SslAuthentication._from_proto(proto_authentication_method)
        elif isinstance(proto_authentication_method, ProtoSasl):
            return SaslAuthentication._from_proto(proto_authentication_method)
        else:
            raise QwakException(
                f"Got unsupported authentication method {proto_authentication_method}"
            )


@dataclass
class PlainAuthentication(BaseAuthentication):
    def _to_proto(self) -> ProtoAuthentication:
        return ProtoAuthentication(plain_configuration=ProtoPlain())

    @classmethod
    def _from_proto(
        cls, proto_authentication_method: ProtoPlain
    ) -> Type["PlainAuthentication"]:
        return cls()


@dataclass
class SslAuthentication(BaseAuthentication):
    def _to_proto(self) -> ProtoAuthentication:
        return ProtoAuthentication(ssl_configuration=ProtoSsl())

    @classmethod
    def _from_proto(
        cls, proto_authentication_method: ProtoSsl
    ) -> Type["SslAuthentication"]:
        return cls()


@dataclass
class SaslAuthentication(BaseAuthentication):
    username_secret: str
    password_secret: str
    sasl_mechanism: SaslMechanism
    security_protocol: SecurityProtocol

    def _to_proto(self) -> ProtoAuthentication:
        self._validate()
        return ProtoAuthentication(
            sasl_configuration=ProtoSasl(
                username_secret=self.username_secret,
                password_secret=self.password_secret,
                sasl_mechanism=self.sasl_mechanism.value,
                security_protocol=self.security_protocol.value,
            )
        )

    @classmethod
    def _from_proto(
        cls, proto_authentication_method: ProtoSasl
    ) -> Type["SaslAuthentication"]:
        return cls(
            username_secret=proto_authentication_method.username_secret,
            password_secret=proto_authentication_method.password_secret,
            sasl_mechanism=SaslMechanism(proto_authentication_method.sasl_mechanism),
            security_protocol=SecurityProtocol(
                proto_authentication_method.security_protocol
            ),
        )

    def _validate(self):
        secret_service_client = SecretServiceClient()

        if not secret_service_client.get_secret(self.username_secret):
            raise QwakException(
                f"Secret for username {self.username_secret} does not exist"
            )
        if not secret_service_client.get_secret(self.password_secret):
            raise QwakException(
                f"Secret for password {self.password_secret} does not exist"
            )



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import google.protobuf.timestamp_pb2
import qwak.offline.serving.v1.feature_values_pb2
import qwak.offline.serving.v1.population_pb2
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _FeatureValuesRequestStatus:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _FeatureValuesRequestStatusEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_FeatureValuesRequestStatus.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    FEATURE_VALUES_REQUEST_STATUS_INVALID: _FeatureValuesRequestStatus.ValueType  # 0
    """Invalid status"""
    FEATURE_VALUES_REQUEST_STATUS_SUCCEEDED: _FeatureValuesRequestStatus.ValueType  # 1
    """Request Success"""
    FEATURE_VALUES_REQUEST_STATUS_PENDING: _FeatureValuesRequestStatus.ValueType  # 2
    """Request Pending"""
    FEATURE_VALUES_REQUEST_STATUS_CANCELLED: _FeatureValuesRequestStatus.ValueType  # 3
    """Request Canceled"""
    FEATURE_VALUES_REQUEST_STATUS_FAILED: _FeatureValuesRequestStatus.ValueType  # 4
    """Request Failed"""

class FeatureValuesRequestStatus(_FeatureValuesRequestStatus, metaclass=_FeatureValuesRequestStatusEnumTypeWrapper): ...

FEATURE_VALUES_REQUEST_STATUS_INVALID: FeatureValuesRequestStatus.ValueType  # 0
"""Invalid status"""
FEATURE_VALUES_REQUEST_STATUS_SUCCEEDED: FeatureValuesRequestStatus.ValueType  # 1
"""Request Success"""
FEATURE_VALUES_REQUEST_STATUS_PENDING: FeatureValuesRequestStatus.ValueType  # 2
"""Request Pending"""
FEATURE_VALUES_REQUEST_STATUS_CANCELLED: FeatureValuesRequestStatus.ValueType  # 3
"""Request Canceled"""
FEATURE_VALUES_REQUEST_STATUS_FAILED: FeatureValuesRequestStatus.ValueType  # 4
"""Request Failed"""
global___FeatureValuesRequestStatus = FeatureValuesRequestStatus

class _FileFormat:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _FileFormatEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_FileFormat.ValueType], builtins.type):  # noqa: F821
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    FILE_FORMAT_INVALID: _FileFormat.ValueType  # 0
    """Invalid format"""
    FILE_FORMAT_CSV: _FileFormat.ValueType  # 1
    """Csv format"""

class FileFormat(_FileFormat, metaclass=_FileFormatEnumTypeWrapper): ...

FILE_FORMAT_INVALID: FileFormat.ValueType  # 0
"""Invalid format"""
FILE_FORMAT_CSV: FileFormat.ValueType  # 1
"""Csv format"""
global___FileFormat = FileFormat

class GetFileUploadUrlRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    POPULATION_FIELD_NUMBER: builtins.int
    @property
    def population(self) -> qwak.offline.serving.v1.population_pb2.PopulationFileUploadUrlType:
        """Used to Get File Upload Url for population dataset"""
    def __init__(
        self,
        *,
        population: qwak.offline.serving.v1.population_pb2.PopulationFileUploadUrlType | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["population", b"population", "type", b"type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["population", b"population", "type", b"type"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["type", b"type"]) -> typing_extensions.Literal["population"] | None: ...

global___GetFileUploadUrlRequest = GetFileUploadUrlRequest

class GetFileUploadUrlResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FILE_UPLOAD_URL_FIELD_NUMBER: builtins.int
    file_upload_url: builtins.str
    """The generated url to which the file can be uploaded"""
    def __init__(
        self,
        *,
        file_upload_url: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["file_upload_url", b"file_upload_url"]) -> None: ...

global___GetFileUploadUrlResponse = GetFileUploadUrlResponse

class GetFeatureValuesRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURES_FIELD_NUMBER: builtins.int
    POPULATION_FIELD_NUMBER: builtins.int
    RESULT_FILE_FORMAT_FIELD_NUMBER: builtins.int
    @property
    def features(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[qwak.offline.serving.v1.feature_values_pb2.FeaturesetFeatures]:
        """a List of featureset name and it's feature names"""
    @property
    def population(self) -> qwak.offline.serving.v1.population_pb2.TimedPopulation:
        """The population dataset with timestamp column"""
    result_file_format: global___FileFormat.ValueType
    """The format of the result files"""
    def __init__(
        self,
        *,
        features: collections.abc.Iterable[qwak.offline.serving.v1.feature_values_pb2.FeaturesetFeatures] | None = ...,
        population: qwak.offline.serving.v1.population_pb2.TimedPopulation | None = ...,
        result_file_format: global___FileFormat.ValueType = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["population", b"population"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["features", b"features", "population", b"population", "result_file_format", b"result_file_format"]) -> None: ...

global___GetFeatureValuesRequest = GetFeatureValuesRequest

class GetFeatureValuesResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    REQUEST_ID_FIELD_NUMBER: builtins.int
    request_id: builtins.str
    """The request_id. used as a handler to retrieve results"""
    def __init__(
        self,
        *,
        request_id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["request_id", b"request_id"]) -> None: ...

global___GetFeatureValuesResponse = GetFeatureValuesResponse

class GetFeatureValuesInRangeRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURES_FIELD_NUMBER: builtins.int
    LOWER_TIME_BOUND_FIELD_NUMBER: builtins.int
    UPPER_TIME_BOUND_FIELD_NUMBER: builtins.int
    POPULATION_FIELD_NUMBER: builtins.int
    RESULT_FILE_FORMAT_FIELD_NUMBER: builtins.int
    @property
    def features(self) -> qwak.offline.serving.v1.feature_values_pb2.FeaturesetFeatures:
        """A single featureset name and it's feature names"""
    @property
    def lower_time_bound(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """Lower time bound for range query"""
    @property
    def upper_time_bound(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """Upper time bound for range query"""
    @property
    def population(self) -> qwak.offline.serving.v1.population_pb2.Population: ...
    result_file_format: global___FileFormat.ValueType
    """The format of the result files"""
    def __init__(
        self,
        *,
        features: qwak.offline.serving.v1.feature_values_pb2.FeaturesetFeatures | None = ...,
        lower_time_bound: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        upper_time_bound: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        population: qwak.offline.serving.v1.population_pb2.Population | None = ...,
        result_file_format: global___FileFormat.ValueType = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["features", b"features", "lower_time_bound", b"lower_time_bound", "optional_population", b"optional_population", "population", b"population", "upper_time_bound", b"upper_time_bound"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["features", b"features", "lower_time_bound", b"lower_time_bound", "optional_population", b"optional_population", "population", b"population", "result_file_format", b"result_file_format", "upper_time_bound", b"upper_time_bound"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["optional_population", b"optional_population"]) -> typing_extensions.Literal["population"] | None: ...

global___GetFeatureValuesInRangeRequest = GetFeatureValuesInRangeRequest

class GetFeatureValuesInRangeResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    REQUEST_ID_FIELD_NUMBER: builtins.int
    request_id: builtins.str
    """The request_id. used as a handler to retrieve results"""
    def __init__(
        self,
        *,
        request_id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["request_id", b"request_id"]) -> None: ...

global___GetFeatureValuesInRangeResponse = GetFeatureValuesInRangeResponse

class GetFeatureValuesResultRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    REQUEST_ID_FIELD_NUMBER: builtins.int
    request_id: builtins.str
    """The request_id handler returned by one of the Get-Feature-Values endpoints"""
    def __init__(
        self,
        *,
        request_id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["request_id", b"request_id"]) -> None: ...

global___GetFeatureValuesResultRequest = GetFeatureValuesResultRequest

class GetFeatureValuesResultResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    STATUS_FIELD_NUMBER: builtins.int
    LINK_TO_FILES_FIELD_NUMBER: builtins.int
    FAILURE_REASON_FIELD_NUMBER: builtins.int
    status: global___FeatureValuesRequestStatus.ValueType
    """The status of the request"""
    @property
    def link_to_files(self) -> global___LinkToFiles: ...
    @property
    def failure_reason(self) -> global___FailureReason: ...
    def __init__(
        self,
        *,
        status: global___FeatureValuesRequestStatus.ValueType = ...,
        link_to_files: global___LinkToFiles | None = ...,
        failure_reason: global___FailureReason | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["failure_reason", b"failure_reason", "link_to_files", b"link_to_files", "payload", b"payload"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["failure_reason", b"failure_reason", "link_to_files", b"link_to_files", "payload", b"payload", "status", b"status"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["payload", b"payload"]) -> typing_extensions.Literal["link_to_files", "failure_reason"] | None: ...

global___GetFeatureValuesResultResponse = GetFeatureValuesResultResponse

class FailureReason(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MESSAGE_FIELD_NUMBER: builtins.int
    message: builtins.str
    """Failure reason"""
    def __init__(
        self,
        *,
        message: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["message", b"message"]) -> None: ...

global___FailureReason = FailureReason

class LinkToFiles(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LINK_TO_FILES_FIELD_NUMBER: builtins.int
    @property
    def link_to_files(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """List of urls to the files"""
    def __init__(
        self,
        *,
        link_to_files: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["link_to_files", b"link_to_files"]) -> None: ...

global___LinkToFiles = LinkToFiles



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/prompt/v1/prompt/prompt_manager_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from _qwak_proto.qwak.prompt.v1.prompt import prompt_pb2 as qwak_dot_prompt_dot_v1_dot_prompt_dot_prompt__pb2
from _qwak_proto.qwak.model_descriptor import open_ai_descriptor_pb2 as qwak_dot_model__descriptor_dot_open__ai__descriptor__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n2qwak/prompt/v1/prompt/prompt_manager_service.proto\x12\x15qwak.prompt.v1.prompt\x1a\"qwak/prompt/v1/prompt/prompt.proto\x1a.qwak/model_descriptor/open_ai_descriptor.proto\"\xa9\x01\n\x13\x43reatePromptRequest\x12\x13\n\x0bprompt_name\x18\x01 \x01(\t\x12\x45\n\x13prompt_version_spec\x18\x02 \x01(\x0b\x32(.qwak.prompt.v1.prompt.PromptVersionSpec\x12\x36\n\x0bprompt_spec\x18\x03 \x01(\x0b\x32!.qwak.prompt.v1.prompt.PromptSpec\"E\n\x14\x43reatePromptResponse\x12-\n\x06prompt\x18\x01 \x01(\x0b\x32\x1d.qwak.prompt.v1.prompt.Prompt\"\x8d\x01\n\x1a\x43reatePromptVersionRequest\x12\x13\n\x0bprompt_name\x18\x01 \x01(\t\x12\x45\n\x13prompt_version_spec\x18\x02 \x01(\x0b\x32(.qwak.prompt.v1.prompt.PromptVersionSpec\x12\x13\n\x0bset_default\x18\x03 \x01(\x08\"[\n\x1b\x43reatePromptVersionResponse\x12<\n\x0eprompt_version\x18\x01 \x01(\x0b\x32$.qwak.prompt.v1.prompt.PromptVersion\"b\n\x13UpdatePromptRequest\x12\x13\n\x0bprompt_name\x18\x01 \x01(\t\x12\x36\n\x0bprompt_spec\x18\x02 \x01(\x0b\x32!.qwak.prompt.v1.prompt.PromptSpec\"\x16\n\x14UpdatePromptResponse\"o\n#GetPromptVersionByPromptNameRequest\x12\x13\n\x0bprompt_name\x18\x01 \x01(\t\x12\x18\n\x0eversion_number\x18\x02 \x01(\x05H\x00\x42\x19\n\x17optional_version_number\"d\n$GetPromptVersionByPromptNameResponse\x12<\n\x0eprompt_version\x18\x01 \x01(\x0b\x32$.qwak.prompt.v1.prompt.PromptVersion\"-\n\x16GetPromptByNameRequest\x12\x13\n\x0bprompt_name\x18\x01 \x01(\t\"H\n\x17GetPromptByNameResponse\x12-\n\x06prompt\x18\x01 \x01(\x0b\x32\x1d.qwak.prompt.v1.prompt.Prompt\"\x14\n\x12ListPromptsRequest\"E\n\x13ListPromptsResponse\x12.\n\x07prompts\x18\x01 \x03(\x0b\x32\x1d.qwak.prompt.v1.prompt.Prompt\"8\n!InitPaginationByPromptNameRequest\x12\x13\n\x0bprompt_name\x18\x01 \x01(\t\"V\n\"InitPaginationByPromptNameResponse\x12\x14\n\x0crecord_count\x18\x01 \x01(\x03\x12\x1a\n\x12max_version_number\x18\x02 \x01(\x05\"\x80\x01\n%ListPromptVersionsByPromptNameRequest\x12\x13\n\x0bprompt_name\x18\x01 \x01(\t\x12\x11\n\tpage_size\x18\x02 \x01(\x05\x12\x13\n\x0bpage_number\x18\x03 \x01(\x05\x12\x1a\n\x12max_version_number\x18\x04 \x01(\x05\"g\n&ListPromptVersionsByPromptNameResponse\x12=\n\x0fprompt_versions\x18\x01 \x03(\x0b\x32$.qwak.prompt.v1.prompt.PromptVersion\"*\n\x13\x44\x65letePromptRequest\x12\x13\n\x0bprompt_name\x18\x01 \x01(\t\"\x16\n\x14\x44\x65letePromptResponse\"I\n\x1a\x44\x65letePromptVersionRequest\x12\x13\n\x0bprompt_name\x18\x01 \x01(\t\x12\x16\n\x0eversion_number\x18\x03 \x01(\x05\"\x1d\n\x1b\x44\x65letePromptVersionResponse\"M\n\x1eSetDefaultPromptVersionRequest\x12\x13\n\x0bprompt_name\x18\x01 \x01(\t\x12\x16\n\x0eversion_number\x18\x02 \x01(\x05\"!\n\x1fSetDefaultPromptVersionResponse\"\x19\n\x17ListPromptBriefsRequest\"U\n\x18ListPromptBriefsResponse\x12\x39\n\rprompt_briefs\x18\x01 \x03(\x0b\x32\".qwak.prompt.v1.prompt.PromptBrief2\xf3\x0b\n\x14PromptManagerService\x12g\n\x0c\x43reatePrompt\x12*.qwak.prompt.v1.prompt.CreatePromptRequest\x1a+.qwak.prompt.v1.prompt.CreatePromptResponse\x12|\n\x13\x43reatePromptVersion\x12\x31.qwak.prompt.v1.prompt.CreatePromptVersionRequest\x1a\x32.qwak.prompt.v1.prompt.CreatePromptVersionResponse\x12g\n\x0cUpdatePrompt\x12*.qwak.prompt.v1.prompt.UpdatePromptRequest\x1a+.qwak.prompt.v1.prompt.UpdatePromptResponse\x12p\n\x0fGetPromptByName\x12-.qwak.prompt.v1.prompt.GetPromptByNameRequest\x1a..qwak.prompt.v1.prompt.GetPromptByNameResponse\x12\x64\n\x0bListPrompts\x12).qwak.prompt.v1.prompt.ListPromptsRequest\x1a*.qwak.prompt.v1.prompt.ListPromptsResponse\x12\x97\x01\n\x1cGetPromptVersionByPromptName\x12:.qwak.prompt.v1.prompt.GetPromptVersionByPromptNameRequest\x1a;.qwak.prompt.v1.prompt.GetPromptVersionByPromptNameResponse\x12\x91\x01\n\x1aInitPaginationByPromptName\x12\x38.qwak.prompt.v1.prompt.InitPaginationByPromptNameRequest\x1a\x39.qwak.prompt.v1.prompt.InitPaginationByPromptNameResponse\x12\x9d\x01\n\x1eListPromptVersionsByPromptName\x12<.qwak.prompt.v1.prompt.ListPromptVersionsByPromptNameRequest\x1a=.qwak.prompt.v1.prompt.ListPromptVersionsByPromptNameResponse\x12g\n\x0c\x44\x65letePrompt\x12*.qwak.prompt.v1.prompt.DeletePromptRequest\x1a+.qwak.prompt.v1.prompt.DeletePromptResponse\x12|\n\x13\x44\x65letePromptVersion\x12\x31.qwak.prompt.v1.prompt.DeletePromptVersionRequest\x1a\x32.qwak.prompt.v1.prompt.DeletePromptVersionResponse\x12\x88\x01\n\x17SetDefaultPromptVersion\x12\x35.qwak.prompt.v1.prompt.SetDefaultPromptVersionRequest\x1a\x36.qwak.prompt.v1.prompt.SetDefaultPromptVersionResponse\x12s\n\x10ListPromptBriefs\x12..qwak.prompt.v1.prompt.ListPromptBriefsRequest\x1a/.qwak.prompt.v1.prompt.ListPromptBriefsResponseB$\n com.qwak.ai.prompt.api.v1.promptP\x01\x62\x06proto3')



_CREATEPROMPTREQUEST = DESCRIPTOR.message_types_by_name['CreatePromptRequest']
_CREATEPROMPTRESPONSE = DESCRIPTOR.message_types_by_name['CreatePromptResponse']
_CREATEPROMPTVERSIONREQUEST = DESCRIPTOR.message_types_by_name['CreatePromptVersionRequest']
_CREATEPROMPTVERSIONRESPONSE = DESCRIPTOR.message_types_by_name['CreatePromptVersionResponse']
_UPDATEPROMPTREQUEST = DESCRIPTOR.message_types_by_name['UpdatePromptRequest']
_UPDATEPROMPTRESPONSE = DESCRIPTOR.message_types_by_name['UpdatePromptResponse']
_GETPROMPTVERSIONBYPROMPTNAMEREQUEST = DESCRIPTOR.message_types_by_name['GetPromptVersionByPromptNameRequest']
_GETPROMPTVERSIONBYPROMPTNAMERESPONSE = DESCRIPTOR.message_types_by_name['GetPromptVersionByPromptNameResponse']
_GETPROMPTBYNAMEREQUEST = DESCRIPTOR.message_types_by_name['GetPromptByNameRequest']
_GETPROMPTBYNAMERESPONSE = DESCRIPTOR.message_types_by_name['GetPromptByNameResponse']
_LISTPROMPTSREQUEST = DESCRIPTOR.message_types_by_name['ListPromptsRequest']
_LISTPROMPTSRESPONSE = DESCRIPTOR.message_types_by_name['ListPromptsResponse']
_INITPAGINATIONBYPROMPTNAMEREQUEST = DESCRIPTOR.message_types_by_name['InitPaginationByPromptNameRequest']
_INITPAGINATIONBYPROMPTNAMERESPONSE = DESCRIPTOR.message_types_by_name['InitPaginationByPromptNameResponse']
_LISTPROMPTVERSIONSBYPROMPTNAMEREQUEST = DESCRIPTOR.message_types_by_name['ListPromptVersionsByPromptNameRequest']
_LISTPROMPTVERSIONSBYPROMPTNAMERESPONSE = DESCRIPTOR.message_types_by_name['ListPromptVersionsByPromptNameResponse']
_DELETEPROMPTREQUEST = DESCRIPTOR.message_types_by_name['DeletePromptRequest']
_DELETEPROMPTRESPONSE = DESCRIPTOR.message_types_by_name['DeletePromptResponse']
_DELETEPROMPTVERSIONREQUEST = DESCRIPTOR.message_types_by_name['DeletePromptVersionRequest']
_DELETEPROMPTVERSIONRESPONSE = DESCRIPTOR.message_types_by_name['DeletePromptVersionResponse']
_SETDEFAULTPROMPTVERSIONREQUEST = DESCRIPTOR.message_types_by_name['SetDefaultPromptVersionRequest']
_SETDEFAULTPROMPTVERSIONRESPONSE = DESCRIPTOR.message_types_by_name['SetDefaultPromptVersionResponse']
_LISTPROMPTBRIEFSREQUEST = DESCRIPTOR.message_types_by_name['ListPromptBriefsRequest']
_LISTPROMPTBRIEFSRESPONSE = DESCRIPTOR.message_types_by_name['ListPromptBriefsResponse']
CreatePromptRequest = _reflection.GeneratedProtocolMessageType('CreatePromptRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATEPROMPTREQUEST,
  '__module__' : 'qwak.prompt.v1.prompt.prompt_manager_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.prompt.v1.prompt.CreatePromptRequest)
  })
_sym_db.RegisterMessage(CreatePromptRequest)

CreatePromptResponse = _reflection.GeneratedProtocolMessageType('CreatePromptResponse', (_message.Message,), {
  'DESCRIPTOR' : _CREATEPROMPTRESPONSE,
  '__module__' : 'qwak.prompt.v1.prompt.prompt_manager_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.prompt.v1.prompt.CreatePromptResponse)
  })
_sym_db.RegisterMessage(CreatePromptResponse)

CreatePromptVersionRequest = _reflection.GeneratedProtocolMessageType('CreatePromptVersionRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATEPROMPTVERSIONREQUEST,
  '__module__' : 'qwak.prompt.v1.prompt.prompt_manager_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.prompt.v1.prompt.CreatePromptVersionRequest)
  })
_sym_db.RegisterMessage(CreatePromptVersionRequest)

CreatePromptVersionResponse = _reflection.GeneratedProtocolMessageType('CreatePromptVersionResponse', (_message.Message,), {
  'DESCRIPTOR' : _CREATEPROMPTVERSIONRESPONSE,
  '__module__' : 'qwak.prompt.v1.prompt.prompt_manager_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.prompt.v1.prompt.CreatePromptVersionResponse)
  })
_sym_db.RegisterMessage(CreatePromptVersionResponse)

UpdatePromptRequest = _reflection.GeneratedProtocolMessageType('UpdatePromptRequest', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEPROMPTREQUEST,
  '__module__' : 'qwak.prompt.v1.prompt.prompt_manager_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.prompt.v1.prompt.UpdatePromptRequest)
  })
_sym_db.RegisterMessage(UpdatePromptRequest)

UpdatePromptResponse = _reflection.GeneratedProtocolMessageType('UpdatePromptResponse', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEPROMPTRESPONSE,
  '__module__' : 'qwak.prompt.v1.prompt.prompt_manager_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.prompt.v1.prompt.UpdatePromptResponse)
  })
_sym_db.RegisterMessage(UpdatePromptResponse)

GetPromptVersionByPromptNameRequest = _reflection.GeneratedProtocolMessageType('GetPromptVersionByPromptNameRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETPROMPTVERSIONBYPROMPTNAMEREQUEST,
  '__module__' : 'qwak.prompt.v1.prompt.prompt_manager_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.prompt.v1.prompt.GetPromptVersionByPromptNameRequest)
  })
_sym_db.RegisterMessage(GetPromptVersionByPromptNameRequest)

GetPromptVersionByPromptNameResponse = _reflection.GeneratedProtocolMessageType('GetPromptVersionByPromptNameResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETPROMPTVERSIONBYPROMPTNAMERESPONSE,
  '__module__' : 'qwak.prompt.v1.prompt.prompt_manager_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.prompt.v1.prompt.GetPromptVersionByPromptNameResponse)
  })
_sym_db.RegisterMessage(GetPromptVersionByPromptNameResponse)

GetPromptByNameRequest = _reflection.GeneratedProtocolMessageType('GetPromptByNameRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETPROMPTBYNAMEREQUEST,
  '__module__' : 'qwak.prompt.v1.prompt.prompt_manager_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.prompt.v1.prompt.GetPromptByNameRequest)
  })
_sym_db.RegisterMessage(GetPromptByNameRequest)

GetPromptByNameResponse = _reflection.GeneratedProtocolMessageType('GetPromptByNameResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETPROMPTBYNAMERESPONSE,
  '__module__' : 'qwak.prompt.v1.prompt.prompt_manager_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.prompt.v1.prompt.GetPromptByNameResponse)
  })
_sym_db.RegisterMessage(GetPromptByNameResponse)

ListPromptsRequest = _reflection.GeneratedProtocolMessageType('ListPromptsRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTPROMPTSREQUEST,
  '__module__' : 'qwak.prompt.v1.prompt.prompt_manager_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.prompt.v1.prompt.ListPromptsRequest)
  })
_sym_db.RegisterMessage(ListPromptsRequest)

ListPromptsResponse = _reflection.GeneratedProtocolMessageType('ListPromptsResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTPROMPTSRESPONSE,
  '__module__' : 'qwak.prompt.v1.prompt.prompt_manager_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.prompt.v1.prompt.ListPromptsResponse)
  })
_sym_db.RegisterMessage(ListPromptsResponse)

InitPaginationByPromptNameRequest = _reflection.GeneratedProtocolMessageType('InitPaginationByPromptNameRequest', (_message.Message,), {
  'DESCRIPTOR' : _INITPAGINATIONBYPROMPTNAMEREQUEST,
  '__module__' : 'qwak.prompt.v1.prompt.prompt_manager_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.prompt.v1.prompt.InitPaginationByPromptNameRequest)
  })
_sym_db.RegisterMessage(InitPaginationByPromptNameRequest)

InitPaginationByPromptNameResponse = _reflection.GeneratedProtocolMessageType('InitPaginationByPromptNameResponse', (_message.Message,), {
  'DESCRIPTOR' : _INITPAGINATIONBYPROMPTNAMERESPONSE,
  '__module__' : 'qwak.prompt.v1.prompt.prompt_manager_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.prompt.v1.prompt.InitPaginationByPromptNameResponse)
  })
_sym_db.RegisterMessage(InitPaginationByPromptNameResponse)

ListPromptVersionsByPromptNameRequest = _reflection.GeneratedProtocolMessageType('ListPromptVersionsByPromptNameRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTPROMPTVERSIONSBYPROMPTNAMEREQUEST,
  '__module__' : 'qwak.prompt.v1.prompt.prompt_manager_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.prompt.v1.prompt.ListPromptVersionsByPromptNameRequest)
  })
_sym_db.RegisterMessage(ListPromptVersionsByPromptNameRequest)

ListPromptVersionsByPromptNameResponse = _reflection.GeneratedProtocolMessageType('ListPromptVersionsByPromptNameResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTPROMPTVERSIONSBYPROMPTNAMERESPONSE,
  '__module__' : 'qwak.prompt.v1.prompt.prompt_manager_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.prompt.v1.prompt.ListPromptVersionsByPromptNameResponse)
  })
_sym_db.RegisterMessage(ListPromptVersionsByPromptNameResponse)

DeletePromptRequest = _reflection.GeneratedProtocolMessageType('DeletePromptRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETEPROMPTREQUEST,
  '__module__' : 'qwak.prompt.v1.prompt.prompt_manager_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.prompt.v1.prompt.DeletePromptRequest)
  })
_sym_db.RegisterMessage(DeletePromptRequest)

DeletePromptResponse = _reflection.GeneratedProtocolMessageType('DeletePromptResponse', (_message.Message,), {
  'DESCRIPTOR' : _DELETEPROMPTRESPONSE,
  '__module__' : 'qwak.prompt.v1.prompt.prompt_manager_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.prompt.v1.prompt.DeletePromptResponse)
  })
_sym_db.RegisterMessage(DeletePromptResponse)

DeletePromptVersionRequest = _reflection.GeneratedProtocolMessageType('DeletePromptVersionRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETEPROMPTVERSIONREQUEST,
  '__module__' : 'qwak.prompt.v1.prompt.prompt_manager_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.prompt.v1.prompt.DeletePromptVersionRequest)
  })
_sym_db.RegisterMessage(DeletePromptVersionRequest)

DeletePromptVersionResponse = _reflection.GeneratedProtocolMessageType('DeletePromptVersionResponse', (_message.Message,), {
  'DESCRIPTOR' : _DELETEPROMPTVERSIONRESPONSE,
  '__module__' : 'qwak.prompt.v1.prompt.prompt_manager_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.prompt.v1.prompt.DeletePromptVersionResponse)
  })
_sym_db.RegisterMessage(DeletePromptVersionResponse)

SetDefaultPromptVersionRequest = _reflection.GeneratedProtocolMessageType('SetDefaultPromptVersionRequest', (_message.Message,), {
  'DESCRIPTOR' : _SETDEFAULTPROMPTVERSIONREQUEST,
  '__module__' : 'qwak.prompt.v1.prompt.prompt_manager_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.prompt.v1.prompt.SetDefaultPromptVersionRequest)
  })
_sym_db.RegisterMessage(SetDefaultPromptVersionRequest)

SetDefaultPromptVersionResponse = _reflection.GeneratedProtocolMessageType('SetDefaultPromptVersionResponse', (_message.Message,), {
  'DESCRIPTOR' : _SETDEFAULTPROMPTVERSIONRESPONSE,
  '__module__' : 'qwak.prompt.v1.prompt.prompt_manager_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.prompt.v1.prompt.SetDefaultPromptVersionResponse)
  })
_sym_db.RegisterMessage(SetDefaultPromptVersionResponse)

ListPromptBriefsRequest = _reflection.GeneratedProtocolMessageType('ListPromptBriefsRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTPROMPTBRIEFSREQUEST,
  '__module__' : 'qwak.prompt.v1.prompt.prompt_manager_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.prompt.v1.prompt.ListPromptBriefsRequest)
  })
_sym_db.RegisterMessage(ListPromptBriefsRequest)

ListPromptBriefsResponse = _reflection.GeneratedProtocolMessageType('ListPromptBriefsResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTPROMPTBRIEFSRESPONSE,
  '__module__' : 'qwak.prompt.v1.prompt.prompt_manager_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.prompt.v1.prompt.ListPromptBriefsResponse)
  })
_sym_db.RegisterMessage(ListPromptBriefsResponse)

_PROMPTMANAGERSERVICE = DESCRIPTOR.services_by_name['PromptManagerService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n com.qwak.ai.prompt.api.v1.promptP\001'
  _CREATEPROMPTREQUEST._serialized_start=162
  _CREATEPROMPTREQUEST._serialized_end=331
  _CREATEPROMPTRESPONSE._serialized_start=333
  _CREATEPROMPTRESPONSE._serialized_end=402
  _CREATEPROMPTVERSIONREQUEST._serialized_start=405
  _CREATEPROMPTVERSIONREQUEST._serialized_end=546
  _CREATEPROMPTVERSIONRESPONSE._serialized_start=548
  _CREATEPROMPTVERSIONRESPONSE._serialized_end=639
  _UPDATEPROMPTREQUEST._serialized_start=641
  _UPDATEPROMPTREQUEST._serialized_end=739
  _UPDATEPROMPTRESPONSE._serialized_start=741
  _UPDATEPROMPTRESPONSE._serialized_end=763
  _GETPROMPTVERSIONBYPROMPTNAMEREQUEST._serialized_start=765
  _GETPROMPTVERSIONBYPROMPTNAMEREQUEST._serialized_end=876
  _GETPROMPTVERSIONBYPROMPTNAMERESPONSE._serialized_start=878
  _GETPROMPTVERSIONBYPROMPTNAMERESPONSE._serialized_end=978
  _GETPROMPTBYNAMEREQUEST._serialized_start=980
  _GETPROMPTBYNAMEREQUEST._serialized_end=1025
  _GETPROMPTBYNAMERESPONSE._serialized_start=1027
  _GETPROMPTBYNAMERESPONSE._serialized_end=1099
  _LISTPROMPTSREQUEST._serialized_start=1101
  _LISTPROMPTSREQUEST._serialized_end=1121
  _LISTPROMPTSRESPONSE._serialized_start=1123
  _LISTPROMPTSRESPONSE._serialized_end=1192
  _INITPAGINATIONBYPROMPTNAMEREQUEST._serialized_start=1194
  _INITPAGINATIONBYPROMPTNAMEREQUEST._serialized_end=1250
  _INITPAGINATIONBYPROMPTNAMERESPONSE._serialized_start=1252
  _INITPAGINATIONBYPROMPTNAMERESPONSE._serialized_end=1338
  _LISTPROMPTVERSIONSBYPROMPTNAMEREQUEST._serialized_start=1341
  _LISTPROMPTVERSIONSBYPROMPTNAMEREQUEST._serialized_end=1469
  _LISTPROMPTVERSIONSBYPROMPTNAMERESPONSE._serialized_start=1471
  _LISTPROMPTVERSIONSBYPROMPTNAMERESPONSE._serialized_end=1574
  _DELETEPROMPTREQUEST._serialized_start=1576
  _DELETEPROMPTREQUEST._serialized_end=1618
  _DELETEPROMPTRESPONSE._serialized_start=1620
  _DELETEPROMPTRESPONSE._serialized_end=1642
  _DELETEPROMPTVERSIONREQUEST._serialized_start=1644
  _DELETEPROMPTVERSIONREQUEST._serialized_end=1717
  _DELETEPROMPTVERSIONRESPONSE._serialized_start=1719
  _DELETEPROMPTVERSIONRESPONSE._serialized_end=1748
  _SETDEFAULTPROMPTVERSIONREQUEST._serialized_start=1750
  _SETDEFAULTPROMPTVERSIONREQUEST._serialized_end=1827
  _SETDEFAULTPROMPTVERSIONRESPONSE._serialized_start=1829
  _SETDEFAULTPROMPTVERSIONRESPONSE._serialized_end=1862
  _LISTPROMPTBRIEFSREQUEST._serialized_start=1864
  _LISTPROMPTBRIEFSREQUEST._serialized_end=1889
  _LISTPROMPTBRIEFSRESPONSE._serialized_start=1891
  _LISTPROMPTBRIEFSRESPONSE._serialized_end=1976
  _PROMPTMANAGERSERVICE._serialized_start=1979
  _PROMPTMANAGERSERVICE._serialized_end=3502
# @@protoc_insertion_point(module_scope)



from base64 import b64decode
from dataclasses import dataclass
from typing import Optional

from _qwak_proto.qwak.admiral.secret.v0.secret_pb2 import SystemSecretValue
from _qwak_proto.qwak.integration.integration_pb2 import Integration
from _qwak_proto.qwak.integration.open_a_i_integration_pb2 import (
    OpenAIApiKeySystemSecretDescriptor,
    OpenAIIntegration,
)
from qwak.clients.administration.eco_system.eco_system_utils import EcosystemUtils
from qwak.clients.system_secret.system_secret_client import SystemSecretClient


@dataclass
class OpenAIApiKeySystemSecret:
    integration_id: str
    integration_name: str
    secret_name: str
    secret_key: str

    @classmethod
    def from_proto(cls, proto: Integration) -> Optional["OpenAIApiKeySystemSecret"]:
        if proto.WhichOneof("type") != "openai_integration":
            raise ValueError(f"Got wrong type of integration: {proto}")
        openai_integration: OpenAIIntegration = proto.openai_integration

        if (
            openai_integration.WhichOneof("auth")
            != "openai_api_key_system_secret_descriptor"
        ):
            return None

        system_secret: OpenAIApiKeySystemSecretDescriptor = (
            openai_integration.openai_api_key_system_secret_descriptor
        )

        return cls(
            integration_id=proto.integration_id,
            integration_name=proto.name,
            secret_name=system_secret.secret_name,
            secret_key=system_secret.api_key_secret_key,
        )

    def get_api_key(self) -> str:
        system_secret_client = SystemSecretClient()
        default_environment_id: str = EcosystemUtils().get_default_environment_id()
        system_secret: SystemSecretValue = system_secret_client.get_system_secret(
            name=self.secret_name, env_id=default_environment_id
        )

        if system_secret.WhichOneof("value") != "opaque_pair":
            raise ValueError("Got unsupported system secret value type")

        encoded_api_key: str = system_secret.opaque_pair.pairs.get(key=self.secret_key)
        return b64decode(encoded_api_key).decode("utf-8")



from dataclasses import dataclass, field
from os.path import expandvars
from typing import Dict, List, Optional, Tuple, Union

import grpc
from _qwak_proto.qwak.builds.build_pb2 import BaseDockerImageType
from _qwak_proto.qwak.builds.builds_pb2 import (
    BuildConfiguration,
    BuildEnvironment,
    BuildModelUri,
    BuildPropertiesProto,
    CondaBuild,
    DockerBuild,
    PoetryBuild,
    PythonBuild,
    RemoteBuild,
    RemoteBuildResources as RemoteBuildResourcesProto,
    Step,
    VirtualEnvironmentBuild,
)
from _qwak_proto.qwak.instance_template.instance_template_pb2 import InstanceType
from qwak.clients.build_orchestrator import BuildOrchestratorClient
from qwak.clients.instance_template.client import InstanceTemplateManagementClient
from qwak.exceptions import QwakException
from qwak.inner.tool.protobuf_factory import protobuf_factory
from qwak.inner.tool.run_config import (
    ConfigCliMap,
    QwakConfigBase,
    YamlConfigMixin,
    validate_bool,
    validate_float,
    validate_int,
    validate_list_of_strings,
    validate_string,
)
from qwak.inner.tool.run_config.utils import (
    validate_positive_int,
    validate_purchase_option,
)
from qwak.model.base import QwakModel

CONFIG_MAPPING: List[ConfigCliMap] = [
    # ConfigV1
    ConfigCliMap(
        "validate_build_artifact", "step.validate_build_artifact", validate_bool, True
    ),
    ConfigCliMap(
        "validate_build_artifact_timeout",
        "step.validate_build_artifact_timeout",
        validate_positive_int,
        True,
    ),
    ConfigCliMap("tests", "step.tests", validate_bool, True),
    # BuildProperties
    ConfigCliMap("model_id", "build_properties.model_id", validate_string, True),
    ConfigCliMap("build_id", "build_properties.build_id", validate_string, False),
    ConfigCliMap(
        "gpu_compatible", "build_properties.gpu_compatible", validate_bool, True
    ),
    ConfigCliMap("branch", "build_properties.branch", validate_string),
    ConfigCliMap("tags", "build_properties.tags", validate_list_of_strings),
    # ModelUri
    ConfigCliMap("uri", "build_properties.model_uri.uri", validate_string, False),
    ConfigCliMap(
        "git_branch", "build_properties.model_uri.git_branch", validate_string
    ),
    ConfigCliMap("main_dir", "build_properties.model_uri.main_dir", validate_string),
    ConfigCliMap(
        "git_credentials", "build_properties.model_uri.git_credentials", validate_string
    ),
    ConfigCliMap(
        "git_secret_ssh", "build_properties.model_uri.git_secret_ssh", validate_string
    ),
    ConfigCliMap(
        "git_credentials_secret",
        "build_properties.model_uri.git_credentials_secret",
        validate_string,
    ),
    ConfigCliMap(
        "dependency_required_folders",
        "build_properties.model_uri.dependency_required_folders",
        validate_list_of_strings,
    ),
    # BuildEnv
    # PythonEnv
    ConfigCliMap(
        "dependency_file_path",
        "build_env.python_env.dependency_file_path",
        validate_string,
    ),
    # DockerConf
    ConfigCliMap("base_image", "build_env.docker.base_image", validate_string),
    ConfigCliMap("param_list", "build_env.docker.params", validate_list_of_strings),
    ConfigCliMap("env_vars", "build_env.docker.env_vars", validate_list_of_strings),
    ConfigCliMap("cache", "build_env.docker.cache", validate_bool),
    ConfigCliMap("push", "build_env.docker.push", validate_bool),
    ConfigCliMap(
        "iam_role_arn", "build_env.docker.assumed_iam_role_arn", validate_string
    ),
    ConfigCliMap(
        "service_account_key_secret_name",
        "build_env.docker.service_account_key_secret_name",
        validate_string,
    ),
    # RemoteConf
    ConfigCliMap("cpus", "build_env.remote.resources.cpus", validate_float),
    ConfigCliMap("memory", "build_env.remote.resources.memory", validate_string),
    ConfigCliMap("gpu_type", "build_env.remote.resources.gpu_type", validate_string),
    ConfigCliMap(
        "gpu_amount", "build_env.remote.resources.gpu_amount", validate_positive_int
    ),
    ConfigCliMap("instance", "build_env.remote.resources.instance", validate_string),
    # Verbosity level
    ConfigCliMap("verbose", "verbose", validate_int),
    ConfigCliMap("deploy", "deploy", validate_bool),
    ConfigCliMap("deployment_instance", "deployment_instance", validate_string),
    ConfigCliMap("purchase_option", "purchase_option", validate_purchase_option),
    ConfigCliMap(
        "provision_instance_timeout",
        "provision_instance_timeout",
        validate_positive_int,
    ),
]


@protobuf_factory(Step)
@dataclass
class Step:
    validate_build_artifact: bool = field(default=True)
    validate_build_artifact_timeout: int = field(default=120)
    tests: bool = field(default=True)


@protobuf_factory(
    BuildModelUri,
    exclude_fields=[
        "git_credentials",
        "git_credentials_secret",
        "dependency_required_folders",
    ],
)
@dataclass
class ModelUri:
    uri: Optional[str] = field(default=None)
    git_branch: Optional[str] = field(default="master")
    main_dir: Optional[str] = field(default="main")
    dependency_required_folders: List[str] = field(default_factory=list)
    git_credentials: Optional[str] = field(default=None)
    git_credentials_secret: Optional[str] = field(default=None)
    git_secret_ssh: Optional[str] = field(default=None)


@protobuf_factory(BuildPropertiesProto)
@dataclass
class BuildProperties:
    build_id: Optional[str] = field(default=None)
    gpu_compatible: bool = field(default=False)
    branch: Optional[str] = field(default="main")
    model_id: str = field(default="")
    tags: List[str] = field(default_factory=list)
    model_uri: ModelUri = field(default_factory=ModelUri)


@protobuf_factory(DockerBuild)
@dataclass
class DockerConf:
    base_image: Optional[str] = field(default=None)
    env_vars: Optional[Union[List[str], Dict[str, str], Tuple]] = field(
        default_factory=list
    )
    assumed_iam_role_arn: str = field(default=None)
    service_account_key_secret_name: str = field(default=None)
    params: Optional[List[str]] = field(default_factory=list)
    no_cache: bool = field(default=False)
    cache: bool = field(default=True)
    push: bool = field(default=True)

    def __post_init__(self):
        if isinstance(self.env_vars, (list, tuple)):
            self.env_vars = [expandvars(var) for var in self.env_vars]


@protobuf_factory(VirtualEnvironmentBuild)
@dataclass
class VirtualenvType:
    python_version: Optional[str] = field(default=None)
    requirements_txt: Optional[str] = field(default=None)


@protobuf_factory(PoetryBuild)
@dataclass
class PoetryType:
    python_version: Optional[str] = field(default=None)
    lock_file: Optional[str] = field(default=None)


@protobuf_factory(CondaBuild)
@dataclass
class CondaType:
    conda_file: Optional[str] = field(default=None)


@protobuf_factory(
    PythonBuild, exclude_fields=["git_credentials_secret", "git_credentials"]
)
@dataclass
class PythonEnv:
    git_credentials: Optional[str] = field(default=None)
    git_credentials_secret: Optional[str] = field(default=None)
    poetry: PoetryType = field(default=None)
    conda: CondaType = field(default=None)
    virtualenv: VirtualenvType = field(default=None)
    dependency_file_path: Optional[str] = field(default=None)

    def __post_init__(self):
        if not (self.poetry or self.conda or self.virtualenv):
            self.conda = CondaType("conda.yml")


@protobuf_factory(RemoteBuildResourcesProto)
@dataclass
class RemoteBuildResources:
    cpus: Optional[float] = field(default=None)
    memory: Optional[str] = field(default=None)
    gpu_type: Optional[str] = field(default=None)
    gpu_amount: Optional[int] = field(default=None)
    instance: Optional[str] = field(default=None)


@protobuf_factory(RemoteBuild)
@dataclass
class RemoteConf:
    is_remote: bool = field(default=True)
    resources: RemoteBuildResources = field(default_factory=RemoteBuildResources)


@protobuf_factory(BuildEnvironment)
@dataclass
class BuildEnv:
    docker: DockerConf = field(default_factory=DockerConf)
    python_env: PythonEnv = field(default_factory=PythonEnv)
    remote: RemoteConf = field(default_factory=RemoteConf)


@protobuf_factory(BuildConfiguration, exclude_fields=["deploy"])
@dataclass
class BuildConfigV1(YamlConfigMixin, QwakConfigBase):
    @property
    def _config_mapping(self) -> List[ConfigCliMap]:
        return CONFIG_MAPPING

    build_properties: BuildProperties = field(default_factory=BuildProperties)
    build_env: Optional[BuildEnv] = field(default_factory=BuildEnv)
    pre_build: Optional[BuildEnv] = field(default=None)
    post_build: Optional[BuildEnv] = field(default=None)
    step: Step = field(default_factory=Step)
    deploy: bool = field(default=False)
    deployment_instance: Optional[str] = field(default=None)
    purchase_option: Optional[str] = field(default=None)
    provision_instance_timeout: Optional[int] = field(default=None)
    verbose: int = field(default=0)
    pre_built_model: Optional[QwakModel] = field(default=None, init=False)

    def _post_merge_cli(self):
        """Implementing post merge actions."""
        self.build_properties.model_id = self.build_properties.model_id.lower()
        self.build_properties.tags = list(self.build_properties.tags)
        self.build_properties.model_uri.dependency_required_folders = list(
            self.build_properties.model_uri.dependency_required_folders
        )
        self.build_env.docker.env_vars = list(self.build_env.docker.env_vars)
        self.build_env.docker.params = list(self.build_env.docker.params)

    def fetch_base_docker_image_name(
        self,
        build_orchestrator: BuildOrchestratorClient,
        instance_template: InstanceTemplateManagementClient,
    ):
        if not self.build_env.docker.base_image:
            is_gpu_used = (
                self.build_env.remote.resources.gpu_type
                and self.build_env.remote.resources.gpu_amount
                and self.build_env.remote.resources.gpu_amount > 0
            )

            if self.build_env.remote.resources.instance and not is_gpu_used:
                is_gpu_used = self.deduce_gpu_from_template(instance_template)

            if is_gpu_used or self.build_properties.gpu_compatible:
                result = build_orchestrator.fetch_base_docker_image_name(
                    BaseDockerImageType.GPU
                )
                self.build_env.docker.base_image = result.base_docker_image_name
            else:
                result = build_orchestrator.fetch_base_docker_image_name(
                    BaseDockerImageType.CPU
                )
                self.build_env.docker.base_image = result.base_docker_image_name

    def deduce_gpu_from_template(self, instance_template):
        try:
            instance_spec = instance_template.get_instance_template(
                self.build_env.remote.resources.instance
            )
            return instance_spec.instance_type == InstanceType.INSTANCE_TYPE_GPU

        except QwakException as e:
            if not e.status_code or e.status_code != grpc.StatusCode.NOT_FOUND:
                raise e



from typing import List

from _qwak_proto.qwak.integration.integration_pb2 import Integration
from _qwak_proto.qwak.integration.integration_service_pb2 import (
    GetIntegrationRequest,
    GetIntegrationResponse,
    ListIntegrationRequest,
    ListIntegrationsResponse,
)
from _qwak_proto.qwak.integration.integration_service_pb2_grpc import (
    IntegrationManagementServiceStub,
)
from dependency_injector.wiring import Provide, inject
from qwak.inner.di_configuration import QwakContainer


class IntegrationManagerClient:
    @inject
    def __init__(self, grpc_channel=Provide[QwakContainer.core_grpc_channel]):
        self._grpc_client: IntegrationManagementServiceStub = (
            IntegrationManagementServiceStub(grpc_channel)
        )

    def list_integrations(self) -> List[Integration]:
        response: ListIntegrationsResponse = self._grpc_client.ListIntegrations(
            ListIntegrationRequest()
        )
        return response.integrations

    def get_by_id(self, integration_id: str) -> Integration:
        response: GetIntegrationResponse = self._grpc_client.GetIntegration(
            GetIntegrationRequest(integration_id=integration_id)
        )
        return response.integration



from collections import defaultdict
from enum import Enum
from typing import Dict, List

from _qwak_proto.qwak.logging.log_line_pb2 import LogLine
from _qwak_proto.qwak.logging.log_reader_service_pb2 import (
    ReadLogsRequest,
    ReadLogsResponse,
)
from _qwak_proto.qwak.logging.log_reader_service_pb2_grpc import (
    LogReaderServiceServicer,
)
from _qwak_proto.qwak.logging.log_source_pb2 import (
    InferenceExecutionSource,
    LogSource,
    ModelRuntimeSource,
    RemoteBuildSource,
    StreamingAggregationFeatureSetSource,
    StreamingFeatureSetSource,
)


class Source(Enum):
    MODEL_RUNTIME = 1
    REMOTE_BUILD = 2
    INFERENCE_EXECUTION = 3
    STREAMING_AGGREGATION = 4
    STREAMING_FEATURE_SET = 5


class MockLogLine:
    def __init__(self, log_line: LogLine, metadata: Dict[str, any]):
        self.log_line = log_line
        self.metadata = metadata


class LoggingServiceApiMock(LogReaderServiceServicer):
    def __init__(self):
        self._logs: Dict[Source, List[MockLogLine]] = defaultdict(list)
        super(LoggingServiceApiMock, self).__init__()

    def ReadLogs(self, request: ReadLogsRequest, context) -> ReadLogsResponse:
        source_to_use = self.__get_source_from_request(request.source)
        logs = []
        if source_to_use == Source.MODEL_RUNTIME:
            logs = self.__read_model_runtime_logs(request)
        elif source_to_use == Source.INFERENCE_EXECUTION:
            logs = self.__read_inference_execution_logs(request)
        elif source_to_use == Source.REMOTE_BUILD:
            logs = self.__read_remote_build_logs(request)
        elif source_to_use == Source.STREAMING_AGGREGATION:
            logs = self.__read_streaming_aggregation_logs(request)
        elif source_to_use == Source.STREAMING_FEATURE_SET:
            logs = self.__read_streaming_feature_set_logs(request)

        if request.after_offset:
            logs = list(
                filter(
                    lambda mocked_log: mocked_log.metadata.get("offset")
                    >= request.after_offset,
                    logs,
                )
            )
        if request.before_offset:
            logs = list(
                filter(
                    lambda mocked_log: mocked_log.metadata.get("offset")
                    < request.before_offset,
                    logs,
                )
            )
        logs.sort(
            key=lambda mocked_log: (
                mocked_log.metadata.get("offset") is None,
                mocked_log.metadata.get("offset"),
            ),
            reverse=True,
        )

        has_next_page = False
        if request.max_number_of_results > 0:
            has_next_page = len(logs) > request.max_number_of_results
            logs = logs[: request.max_number_of_results]
        try:
            first_offset = min(
                list(map(lambda mocked_log: mocked_log.metadata.get("offset"), logs))
            )
            last_offset = max(
                list(map(lambda mocked_log: mocked_log.metadata.get("offset"), logs))
            )
        except Exception:
            first_offset = "0"
            last_offset = "0"

        return ReadLogsResponse(
            log_line=list(map(lambda mocked_log: mocked_log.log_line, logs)),
            first_offset=first_offset,
            last_offset=last_offset,
            has_next_page=has_next_page,
        )

    @staticmethod
    def _get_source(source: LogSource):
        if type(source) is ModelRuntimeSource:
            return Source.MODEL_RUNTIME
        elif type(source) is RemoteBuildSource:
            return Source.REMOTE_BUILD
        elif type(source) is InferenceExecutionSource:
            return Source.INFERENCE_EXECUTION
        elif type(source) is StreamingFeatureSetSource:
            return Source.STREAMING_AGGREGATION
        elif type(source) is StreamingAggregationFeatureSetSource:
            return Source.STREAMING_FEATURE_SET
        else:
            raise Exception("Unknown source")

    @staticmethod
    def __get_source_from_request(source: LogSource):
        request_source = source.WhichOneof("source")
        if request_source == "model_runtime":
            return Source.MODEL_RUNTIME
        elif request_source == "remote_build":
            return Source.REMOTE_BUILD
        elif request_source == "inference_execution":
            return Source.INFERENCE_EXECUTION
        elif request_source == "streaming_aggregation_feature_set":
            return Source.STREAMING_AGGREGATION
        elif request_source == "streaming_feature_set":
            return Source.STREAMING_FEATURE_SET
        else:
            raise Exception("Unknown source")

    def __read_model_runtime_logs(self, request: ReadLogsRequest) -> List[MockLogLine]:
        logs: List[MockLogLine] = self._logs[Source.MODEL_RUNTIME]
        if request.source.model_runtime.deployment_id:
            logs = list(
                filter(
                    lambda mocked_log: mocked_log.metadata.get("deployment_id")
                    == request.source.model_runtime.deployment_id,
                    logs,
                )
            )
        elif request.source.model_runtime.build_id:
            logs = list(
                filter(
                    lambda mocked_log: mocked_log.metadata.get("build_id")
                    == request.source.model_runtime.build_id,
                    logs,
                )
            )

        return logs

    def __read_inference_execution_logs(self, request) -> List[MockLogLine]:
        logs: List[MockLogLine] = self._logs[Source.INFERENCE_EXECUTION]
        if request.source.inference_execution.inference_job_id:
            logs = list(
                filter(
                    lambda mocked_log: mocked_log.metadata.get("inference_job_id")
                    == request.source.inference_execution.inference_job_id,
                    logs,
                )
            )
        elif request.source.inference_execution.inference_task_id:
            logs = list(
                filter(
                    lambda mocked_log: mocked_log.metadata.get("inference_task_id")
                    == request.source.inference_execution.inference_task_id,
                    logs,
                )
            )

        return logs

    def __read_remote_build_logs(self, request) -> List[MockLogLine]:
        logs: List[MockLogLine] = self._logs[Source.REMOTE_BUILD]
        if request.source.remote_build.build_id:
            logs = list(
                filter(
                    lambda mocked_log: mocked_log.metadata.get("build_id")
                    == request.source.remote_build.build_id,
                    logs,
                )
            )
        if request.source.remote_build.phase_ids:
            logs = list(
                filter(
                    lambda mocked_log: mocked_log.metadata.get("phase_id")
                    in request.source.remote_build.phase_ids,
                    logs,
                )
            )

        return logs

    def __read_streaming_aggregation_logs(self, request):
        raise Exception("Unimplemented")

    def __read_streaming_feature_set_logs(self, request):
        raise Exception("Unimplemented")

    def add_mock_logs(self, source: LogSource, rows: List[MockLogLine]):
        self._logs[self._get_source(source)].extend(rows)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/builds/internal_builds_orchestrator_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from _qwak_proto.qwak.builds import build_pb2 as qwak_dot_builds_dot_build__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n6qwak/builds/internal_builds_orchestrator_service.proto\x12\x18qwak.builds.orchestrator\x1a\x17qwak/builds/build.proto\"\x8e\x01\n\x10\x42uildInitDetails\x12=\n\nbuild_spec\x18\x01 \x01(\x0b\x32).qwak.builds.orchestrator.RemoteBuildSpec\x12;\n\tinitiator\x18\x02 \x01(\x0b\x32(.qwak.builds.orchestrator.BuildInitiator\"c\n\x19InternalBuildModelRequest\x12\x46\n\x12\x62uild_init_details\x18\x01 \x01(\x0b\x32*.qwak.builds.orchestrator.BuildInitDetails\"\x1c\n\x1aInternalBuildModelResponse2\x9c\x01\n!InternalBuildsOrchestratorService\x12w\n\nBuildModel\x12\x33.qwak.builds.orchestrator.InternalBuildModelRequest\x1a\x34.qwak.builds.orchestrator.InternalBuildModelResponseB\'\n#com.qwak.ai.builds.orchestrator.apiP\x01\x62\x06proto3')



_BUILDINITDETAILS = DESCRIPTOR.message_types_by_name['BuildInitDetails']
_INTERNALBUILDMODELREQUEST = DESCRIPTOR.message_types_by_name['InternalBuildModelRequest']
_INTERNALBUILDMODELRESPONSE = DESCRIPTOR.message_types_by_name['InternalBuildModelResponse']
BuildInitDetails = _reflection.GeneratedProtocolMessageType('BuildInitDetails', (_message.Message,), {
  'DESCRIPTOR' : _BUILDINITDETAILS,
  '__module__' : 'qwak.builds.internal_builds_orchestrator_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.builds.orchestrator.BuildInitDetails)
  })
_sym_db.RegisterMessage(BuildInitDetails)

InternalBuildModelRequest = _reflection.GeneratedProtocolMessageType('InternalBuildModelRequest', (_message.Message,), {
  'DESCRIPTOR' : _INTERNALBUILDMODELREQUEST,
  '__module__' : 'qwak.builds.internal_builds_orchestrator_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.builds.orchestrator.InternalBuildModelRequest)
  })
_sym_db.RegisterMessage(InternalBuildModelRequest)

InternalBuildModelResponse = _reflection.GeneratedProtocolMessageType('InternalBuildModelResponse', (_message.Message,), {
  'DESCRIPTOR' : _INTERNALBUILDMODELRESPONSE,
  '__module__' : 'qwak.builds.internal_builds_orchestrator_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.builds.orchestrator.InternalBuildModelResponse)
  })
_sym_db.RegisterMessage(InternalBuildModelResponse)

_INTERNALBUILDSORCHESTRATORSERVICE = DESCRIPTOR.services_by_name['InternalBuildsOrchestratorService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n#com.qwak.ai.builds.orchestrator.apiP\001'
  _BUILDINITDETAILS._serialized_start=110
  _BUILDINITDETAILS._serialized_end=252
  _INTERNALBUILDMODELREQUEST._serialized_start=254
  _INTERNALBUILDMODELREQUEST._serialized_end=353
  _INTERNALBUILDMODELRESPONSE._serialized_start=355
  _INTERNALBUILDMODELRESPONSE._serialized_end=383
  _INTERNALBUILDSORCHESTRATORSERVICE._serialized_start=386
  _INTERNALBUILDSORCHESTRATORSERVICE._serialized_end=542
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/self_service/account/v0/account_membership_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from _qwak_proto.qwak.self_service.account.v0 import account_membership_pb2 as qwak_dot_self__service_dot_account_dot_v0_dot_account__membership__pb2
from _qwak_proto.qwak.self_service.user.v1 import user_pb2 as qwak_dot_self__service_dot_user_dot_v1_dot_user__pb2
from _qwak_proto.qwak.administration.v0.users import user_pb2 as qwak_dot_administration_dot_v0_dot_users_dot_user__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n=qwak/self_service/account/v0/account_membership_service.proto\x12\x1cqwak.self_service.account.v0\x1a\x35qwak/self_service/account/v0/account_membership.proto\x1a$qwak/self_service/user/v1/user.proto\x1a\'qwak/administration/v0/users/user.proto\" \n\x1eListAccountUserProfilesRequest\"`\n\x1fListAccountUserProfilesResponse\x12=\n\ruser_profiles\x18\x01 \x03(\x0b\x32&.qwak.self_service.user.v1.UserProfile\"d\n\x15UpdateUserRoleRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12?\n\x0c\x61\x63\x63ount_role\x18\x02 \x01(\x0e\x32).qwak.administration.user.QwakAccountRole\"V\n\x16UpdateUserRoleResponse\x12<\n\x0cuser_profile\x18\x01 \x01(\x0b\x32&.qwak.self_service.user.v1.UserProfile\"L\n%UpdateUserAssignedEnvironmentsRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12\x17\n\x0f\x65nvironment_ids\x18\x02 \x03(\t\"f\n&UpdateUserAssignedEnvironmentsResponse\x12<\n\x0cuser_profile\x18\x01 \x01(\x0b\x32&.qwak.self_service.user.v1.UserProfile\"*\n\x1cRemoveUserFromAccountRequest\x12\n\n\x02id\x18\x01 \x01(\t\"\x1f\n\x1dRemoveUserFromAccountResponse\"Z\n\x1aInviteToJoinAccountRequest\x12<\n\x07options\x18\x01 \x03(\x0b\x32+.qwak.self_service.account.v0.InviteOptions\"n\n\x1bInviteToJoinAccountResponse\x12O\n\x0binvitations\x18\x01 \x03(\x0b\x32:.qwak.self_service.account.v0.JoiningInvitationDescription\"\x8a\x01\n\x17UpdateInvitationRequest\x12\x15\n\rinvitation_id\x18\x01 \x01(\t\x12?\n\x0c\x61\x63\x63ount_role\x18\x02 \x01(\x0e\x32).qwak.administration.user.QwakAccountRole\x12\x17\n\x0f\x65nvironment_ids\x18\x03 \x03(\t\"j\n\x18UpdateInvitationResponse\x12N\n\ninvitation\x18\x01 \x01(\x0b\x32:.qwak.self_service.account.v0.JoiningInvitationDescription\"0\n\x17\x41\x63\x63\x65ptInvitationRequest\x12\x15\n\rinvitation_id\x18\x01 \x01(\t\"\x1a\n\x18\x41\x63\x63\x65ptInvitationResponse\"7\n\x1e\x43\x61ncelJoiningInvitationRequest\x12\x15\n\rinvitation_id\x18\x01 \x01(\t\"!\n\x1f\x43\x61ncelJoiningInvitationResponse\"\x1f\n\x1dListAccountInvitationsRequest\"q\n\x1eListAccountInvitationsResponse\x12O\n\x0binvitations\x18\x01 \x03(\x0b\x32:.qwak.self_service.account.v0.JoiningInvitationDescription\"\x1c\n\x1aListUserInvitationsRequest\"n\n\x1bListUserInvitationsResponse\x12O\n\x0binvitations\x18\x01 \x03(\x0b\x32:.qwak.self_service.account.v0.JoiningInvitationDescription\"\x1c\n\x1aListUserMembershipsRequest\"\xbb\x01\n\x1bListUserMembershipsResponse\x12K\n\ruser_accounts\x18\x01 \x03(\x0b\x32\x34.qwak.self_service.account.v0.UserAccountDescription\x12O\n\x0binvitations\x18\x02 \x03(\x0b\x32:.qwak.self_service.account.v0.JoiningInvitationDescription2\xcf\x0c\n\x18\x41\x63\x63ountMembershipService\x12\x96\x01\n\x17ListAccountUserProfiles\x12<.qwak.self_service.account.v0.ListAccountUserProfilesRequest\x1a=.qwak.self_service.account.v0.ListAccountUserProfilesResponse\x12{\n\x0eUpdateUserRole\x12\x33.qwak.self_service.account.v0.UpdateUserRoleRequest\x1a\x34.qwak.self_service.account.v0.UpdateUserRoleResponse\x12\xab\x01\n\x1eUpdateUserAssignedEnvironments\x12\x43.qwak.self_service.account.v0.UpdateUserAssignedEnvironmentsRequest\x1a\x44.qwak.self_service.account.v0.UpdateUserAssignedEnvironmentsResponse\x12\x90\x01\n\x15RemoveUserFromAccount\x12:.qwak.self_service.account.v0.RemoveUserFromAccountRequest\x1a;.qwak.self_service.account.v0.RemoveUserFromAccountResponse\x12\x8a\x01\n\x13InviteToJoinAccount\x12\x38.qwak.self_service.account.v0.InviteToJoinAccountRequest\x1a\x39.qwak.self_service.account.v0.InviteToJoinAccountResponse\x12\x81\x01\n\x10UpdateInvitation\x12\x35.qwak.self_service.account.v0.UpdateInvitationRequest\x1a\x36.qwak.self_service.account.v0.UpdateInvitationResponse\x12\x81\x01\n\x10\x41\x63\x63\x65ptInvitation\x12\x35.qwak.self_service.account.v0.AcceptInvitationRequest\x1a\x36.qwak.self_service.account.v0.AcceptInvitationResponse\x12\x96\x01\n\x17\x43\x61ncelJoiningInvitation\x12<.qwak.self_service.account.v0.CancelJoiningInvitationRequest\x1a=.qwak.self_service.account.v0.CancelJoiningInvitationResponse\x12\x93\x01\n\x16ListAccountInvitations\x12;.qwak.self_service.account.v0.ListAccountInvitationsRequest\x1a<.qwak.self_service.account.v0.ListAccountInvitationsResponse\x12\x8a\x01\n\x13ListUserInvitations\x12\x38.qwak.self_service.account.v0.ListUserInvitationsRequest\x1a\x39.qwak.self_service.account.v0.ListUserInvitationsResponse\x12\x8a\x01\n\x13ListUserMemberships\x12\x38.qwak.self_service.account.v0.ListUserMembershipsRequest\x1a\x39.qwak.self_service.account.v0.ListUserMembershipsResponseB\xb9\x01\n#com.qwak.ai.self_service.account.v0P\x01Z\x8f\x01github.com/qwak-ai/qwak-platform/services/core/java/user-management/user-management-api/pb/qwak/self_service/account/v0;self_service_account_v0b\x06proto3')



_LISTACCOUNTUSERPROFILESREQUEST = DESCRIPTOR.message_types_by_name['ListAccountUserProfilesRequest']
_LISTACCOUNTUSERPROFILESRESPONSE = DESCRIPTOR.message_types_by_name['ListAccountUserProfilesResponse']
_UPDATEUSERROLEREQUEST = DESCRIPTOR.message_types_by_name['UpdateUserRoleRequest']
_UPDATEUSERROLERESPONSE = DESCRIPTOR.message_types_by_name['UpdateUserRoleResponse']
_UPDATEUSERASSIGNEDENVIRONMENTSREQUEST = DESCRIPTOR.message_types_by_name['UpdateUserAssignedEnvironmentsRequest']
_UPDATEUSERASSIGNEDENVIRONMENTSRESPONSE = DESCRIPTOR.message_types_by_name['UpdateUserAssignedEnvironmentsResponse']
_REMOVEUSERFROMACCOUNTREQUEST = DESCRIPTOR.message_types_by_name['RemoveUserFromAccountRequest']
_REMOVEUSERFROMACCOUNTRESPONSE = DESCRIPTOR.message_types_by_name['RemoveUserFromAccountResponse']
_INVITETOJOINACCOUNTREQUEST = DESCRIPTOR.message_types_by_name['InviteToJoinAccountRequest']
_INVITETOJOINACCOUNTRESPONSE = DESCRIPTOR.message_types_by_name['InviteToJoinAccountResponse']
_UPDATEINVITATIONREQUEST = DESCRIPTOR.message_types_by_name['UpdateInvitationRequest']
_UPDATEINVITATIONRESPONSE = DESCRIPTOR.message_types_by_name['UpdateInvitationResponse']
_ACCEPTINVITATIONREQUEST = DESCRIPTOR.message_types_by_name['AcceptInvitationRequest']
_ACCEPTINVITATIONRESPONSE = DESCRIPTOR.message_types_by_name['AcceptInvitationResponse']
_CANCELJOININGINVITATIONREQUEST = DESCRIPTOR.message_types_by_name['CancelJoiningInvitationRequest']
_CANCELJOININGINVITATIONRESPONSE = DESCRIPTOR.message_types_by_name['CancelJoiningInvitationResponse']
_LISTACCOUNTINVITATIONSREQUEST = DESCRIPTOR.message_types_by_name['ListAccountInvitationsRequest']
_LISTACCOUNTINVITATIONSRESPONSE = DESCRIPTOR.message_types_by_name['ListAccountInvitationsResponse']
_LISTUSERINVITATIONSREQUEST = DESCRIPTOR.message_types_by_name['ListUserInvitationsRequest']
_LISTUSERINVITATIONSRESPONSE = DESCRIPTOR.message_types_by_name['ListUserInvitationsResponse']
_LISTUSERMEMBERSHIPSREQUEST = DESCRIPTOR.message_types_by_name['ListUserMembershipsRequest']
_LISTUSERMEMBERSHIPSRESPONSE = DESCRIPTOR.message_types_by_name['ListUserMembershipsResponse']
ListAccountUserProfilesRequest = _reflection.GeneratedProtocolMessageType('ListAccountUserProfilesRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTACCOUNTUSERPROFILESREQUEST,
  '__module__' : 'qwak.self_service.account.v0.account_membership_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.self_service.account.v0.ListAccountUserProfilesRequest)
  })
_sym_db.RegisterMessage(ListAccountUserProfilesRequest)

ListAccountUserProfilesResponse = _reflection.GeneratedProtocolMessageType('ListAccountUserProfilesResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTACCOUNTUSERPROFILESRESPONSE,
  '__module__' : 'qwak.self_service.account.v0.account_membership_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.self_service.account.v0.ListAccountUserProfilesResponse)
  })
_sym_db.RegisterMessage(ListAccountUserProfilesResponse)

UpdateUserRoleRequest = _reflection.GeneratedProtocolMessageType('UpdateUserRoleRequest', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEUSERROLEREQUEST,
  '__module__' : 'qwak.self_service.account.v0.account_membership_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.self_service.account.v0.UpdateUserRoleRequest)
  })
_sym_db.RegisterMessage(UpdateUserRoleRequest)

UpdateUserRoleResponse = _reflection.GeneratedProtocolMessageType('UpdateUserRoleResponse', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEUSERROLERESPONSE,
  '__module__' : 'qwak.self_service.account.v0.account_membership_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.self_service.account.v0.UpdateUserRoleResponse)
  })
_sym_db.RegisterMessage(UpdateUserRoleResponse)

UpdateUserAssignedEnvironmentsRequest = _reflection.GeneratedProtocolMessageType('UpdateUserAssignedEnvironmentsRequest', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEUSERASSIGNEDENVIRONMENTSREQUEST,
  '__module__' : 'qwak.self_service.account.v0.account_membership_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.self_service.account.v0.UpdateUserAssignedEnvironmentsRequest)
  })
_sym_db.RegisterMessage(UpdateUserAssignedEnvironmentsRequest)

UpdateUserAssignedEnvironmentsResponse = _reflection.GeneratedProtocolMessageType('UpdateUserAssignedEnvironmentsResponse', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEUSERASSIGNEDENVIRONMENTSRESPONSE,
  '__module__' : 'qwak.self_service.account.v0.account_membership_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.self_service.account.v0.UpdateUserAssignedEnvironmentsResponse)
  })
_sym_db.RegisterMessage(UpdateUserAssignedEnvironmentsResponse)

RemoveUserFromAccountRequest = _reflection.GeneratedProtocolMessageType('RemoveUserFromAccountRequest', (_message.Message,), {
  'DESCRIPTOR' : _REMOVEUSERFROMACCOUNTREQUEST,
  '__module__' : 'qwak.self_service.account.v0.account_membership_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.self_service.account.v0.RemoveUserFromAccountRequest)
  })
_sym_db.RegisterMessage(RemoveUserFromAccountRequest)

RemoveUserFromAccountResponse = _reflection.GeneratedProtocolMessageType('RemoveUserFromAccountResponse', (_message.Message,), {
  'DESCRIPTOR' : _REMOVEUSERFROMACCOUNTRESPONSE,
  '__module__' : 'qwak.self_service.account.v0.account_membership_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.self_service.account.v0.RemoveUserFromAccountResponse)
  })
_sym_db.RegisterMessage(RemoveUserFromAccountResponse)

InviteToJoinAccountRequest = _reflection.GeneratedProtocolMessageType('InviteToJoinAccountRequest', (_message.Message,), {
  'DESCRIPTOR' : _INVITETOJOINACCOUNTREQUEST,
  '__module__' : 'qwak.self_service.account.v0.account_membership_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.self_service.account.v0.InviteToJoinAccountRequest)
  })
_sym_db.RegisterMessage(InviteToJoinAccountRequest)

InviteToJoinAccountResponse = _reflection.GeneratedProtocolMessageType('InviteToJoinAccountResponse', (_message.Message,), {
  'DESCRIPTOR' : _INVITETOJOINACCOUNTRESPONSE,
  '__module__' : 'qwak.self_service.account.v0.account_membership_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.self_service.account.v0.InviteToJoinAccountResponse)
  })
_sym_db.RegisterMessage(InviteToJoinAccountResponse)

UpdateInvitationRequest = _reflection.GeneratedProtocolMessageType('UpdateInvitationRequest', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEINVITATIONREQUEST,
  '__module__' : 'qwak.self_service.account.v0.account_membership_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.self_service.account.v0.UpdateInvitationRequest)
  })
_sym_db.RegisterMessage(UpdateInvitationRequest)

UpdateInvitationResponse = _reflection.GeneratedProtocolMessageType('UpdateInvitationResponse', (_message.Message,), {
  'DESCRIPTOR' : _UPDATEINVITATIONRESPONSE,
  '__module__' : 'qwak.self_service.account.v0.account_membership_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.self_service.account.v0.UpdateInvitationResponse)
  })
_sym_db.RegisterMessage(UpdateInvitationResponse)

AcceptInvitationRequest = _reflection.GeneratedProtocolMessageType('AcceptInvitationRequest', (_message.Message,), {
  'DESCRIPTOR' : _ACCEPTINVITATIONREQUEST,
  '__module__' : 'qwak.self_service.account.v0.account_membership_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.self_service.account.v0.AcceptInvitationRequest)
  })
_sym_db.RegisterMessage(AcceptInvitationRequest)

AcceptInvitationResponse = _reflection.GeneratedProtocolMessageType('AcceptInvitationResponse', (_message.Message,), {
  'DESCRIPTOR' : _ACCEPTINVITATIONRESPONSE,
  '__module__' : 'qwak.self_service.account.v0.account_membership_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.self_service.account.v0.AcceptInvitationResponse)
  })
_sym_db.RegisterMessage(AcceptInvitationResponse)

CancelJoiningInvitationRequest = _reflection.GeneratedProtocolMessageType('CancelJoiningInvitationRequest', (_message.Message,), {
  'DESCRIPTOR' : _CANCELJOININGINVITATIONREQUEST,
  '__module__' : 'qwak.self_service.account.v0.account_membership_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.self_service.account.v0.CancelJoiningInvitationRequest)
  })
_sym_db.RegisterMessage(CancelJoiningInvitationRequest)

CancelJoiningInvitationResponse = _reflection.GeneratedProtocolMessageType('CancelJoiningInvitationResponse', (_message.Message,), {
  'DESCRIPTOR' : _CANCELJOININGINVITATIONRESPONSE,
  '__module__' : 'qwak.self_service.account.v0.account_membership_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.self_service.account.v0.CancelJoiningInvitationResponse)
  })
_sym_db.RegisterMessage(CancelJoiningInvitationResponse)

ListAccountInvitationsRequest = _reflection.GeneratedProtocolMessageType('ListAccountInvitationsRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTACCOUNTINVITATIONSREQUEST,
  '__module__' : 'qwak.self_service.account.v0.account_membership_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.self_service.account.v0.ListAccountInvitationsRequest)
  })
_sym_db.RegisterMessage(ListAccountInvitationsRequest)

ListAccountInvitationsResponse = _reflection.GeneratedProtocolMessageType('ListAccountInvitationsResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTACCOUNTINVITATIONSRESPONSE,
  '__module__' : 'qwak.self_service.account.v0.account_membership_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.self_service.account.v0.ListAccountInvitationsResponse)
  })
_sym_db.RegisterMessage(ListAccountInvitationsResponse)

ListUserInvitationsRequest = _reflection.GeneratedProtocolMessageType('ListUserInvitationsRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTUSERINVITATIONSREQUEST,
  '__module__' : 'qwak.self_service.account.v0.account_membership_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.self_service.account.v0.ListUserInvitationsRequest)
  })
_sym_db.RegisterMessage(ListUserInvitationsRequest)

ListUserInvitationsResponse = _reflection.GeneratedProtocolMessageType('ListUserInvitationsResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTUSERINVITATIONSRESPONSE,
  '__module__' : 'qwak.self_service.account.v0.account_membership_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.self_service.account.v0.ListUserInvitationsResponse)
  })
_sym_db.RegisterMessage(ListUserInvitationsResponse)

ListUserMembershipsRequest = _reflection.GeneratedProtocolMessageType('ListUserMembershipsRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTUSERMEMBERSHIPSREQUEST,
  '__module__' : 'qwak.self_service.account.v0.account_membership_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.self_service.account.v0.ListUserMembershipsRequest)
  })
_sym_db.RegisterMessage(ListUserMembershipsRequest)

ListUserMembershipsResponse = _reflection.GeneratedProtocolMessageType('ListUserMembershipsResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTUSERMEMBERSHIPSRESPONSE,
  '__module__' : 'qwak.self_service.account.v0.account_membership_service_pb2'
  # @@protoc_insertion_point(class_scope:qwak.self_service.account.v0.ListUserMembershipsResponse)
  })
_sym_db.RegisterMessage(ListUserMembershipsResponse)

_ACCOUNTMEMBERSHIPSERVICE = DESCRIPTOR.services_by_name['AccountMembershipService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n#com.qwak.ai.self_service.account.v0P\001Z\217\001github.com/qwak-ai/qwak-platform/services/core/java/user-management/user-management-api/pb/qwak/self_service/account/v0;self_service_account_v0'
  _LISTACCOUNTUSERPROFILESREQUEST._serialized_start=229
  _LISTACCOUNTUSERPROFILESREQUEST._serialized_end=261
  _LISTACCOUNTUSERPROFILESRESPONSE._serialized_start=263
  _LISTACCOUNTUSERPROFILESRESPONSE._serialized_end=359
  _UPDATEUSERROLEREQUEST._serialized_start=361
  _UPDATEUSERROLEREQUEST._serialized_end=461
  _UPDATEUSERROLERESPONSE._serialized_start=463
  _UPDATEUSERROLERESPONSE._serialized_end=549
  _UPDATEUSERASSIGNEDENVIRONMENTSREQUEST._serialized_start=551
  _UPDATEUSERASSIGNEDENVIRONMENTSREQUEST._serialized_end=627
  _UPDATEUSERASSIGNEDENVIRONMENTSRESPONSE._serialized_start=629
  _UPDATEUSERASSIGNEDENVIRONMENTSRESPONSE._serialized_end=731
  _REMOVEUSERFROMACCOUNTREQUEST._serialized_start=733
  _REMOVEUSERFROMACCOUNTREQUEST._serialized_end=775
  _REMOVEUSERFROMACCOUNTRESPONSE._serialized_start=777
  _REMOVEUSERFROMACCOUNTRESPONSE._serialized_end=808
  _INVITETOJOINACCOUNTREQUEST._serialized_start=810
  _INVITETOJOINACCOUNTREQUEST._serialized_end=900
  _INVITETOJOINACCOUNTRESPONSE._serialized_start=902
  _INVITETOJOINACCOUNTRESPONSE._serialized_end=1012
  _UPDATEINVITATIONREQUEST._serialized_start=1015
  _UPDATEINVITATIONREQUEST._serialized_end=1153
  _UPDATEINVITATIONRESPONSE._serialized_start=1155
  _UPDATEINVITATIONRESPONSE._serialized_end=1261
  _ACCEPTINVITATIONREQUEST._serialized_start=1263
  _ACCEPTINVITATIONREQUEST._serialized_end=1311
  _ACCEPTINVITATIONRESPONSE._serialized_start=1313
  _ACCEPTINVITATIONRESPONSE._serialized_end=1339
  _CANCELJOININGINVITATIONREQUEST._serialized_start=1341
  _CANCELJOININGINVITATIONREQUEST._serialized_end=1396
  _CANCELJOININGINVITATIONRESPONSE._serialized_start=1398
  _CANCELJOININGINVITATIONRESPONSE._serialized_end=1431
  _LISTACCOUNTINVITATIONSREQUEST._serialized_start=1433
  _LISTACCOUNTINVITATIONSREQUEST._serialized_end=1464
  _LISTACCOUNTINVITATIONSRESPONSE._serialized_start=1466
  _LISTACCOUNTINVITATIONSRESPONSE._serialized_end=1579
  _LISTUSERINVITATIONSREQUEST._serialized_start=1581
  _LISTUSERINVITATIONSREQUEST._serialized_end=1609
  _LISTUSERINVITATIONSRESPONSE._serialized_start=1611
  _LISTUSERINVITATIONSRESPONSE._serialized_end=1721
  _LISTUSERMEMBERSHIPSREQUEST._serialized_start=1723
  _LISTUSERMEMBERSHIPSREQUEST._serialized_end=1751
  _LISTUSERMEMBERSHIPSRESPONSE._serialized_start=1754
  _LISTUSERMEMBERSHIPSRESPONSE._serialized_end=1941
  _ACCOUNTMEMBERSHIPSERVICE._serialized_start=1944
  _ACCOUNTMEMBERSHIPSERVICE._serialized_end=3559
# @@protoc_insertion_point(module_scope)



import uuid
from pathlib import Path

import grpc

from qwak.clients.secret_service import SecretServiceClient
from qwak.exceptions import (
    QwakSuggestionException,
    QwakException,
)
from qwak.inner.build_logic.constants.host_resource import HOST_TEMP_BUILD_DIR
from qwak.inner.build_logic.interface.step_inteface import Step
from qwak.inner.build_logic.tools.dependencies_tools import find_dependency_files
from qwak.inner.build_logic.tools.text import snake_case
from qwak.inner.instance_template.verify_template_id import verify_template_id
from qwak.inner.provider import Provider
from qwak.model.base import QwakModel


class PreFetchValidationStep(Step):
    STEP_DESCRIPTION = "Pre model fetch validation"
    DEFAULT_CPUS = 2
    DEFAULT_MEMORY = "4Gi"
    VALIDATION_FAILURE_CPU_AND_GPU_MESSAGE = (
        "CPU and GPU configured together, Invalid configuration."
    )
    VALIDATION_FAILURE_CPU_AND_GPU_SUGGESTION = "Please configure only CPU or GPU"
    VALIDATION_FAILURE_CPU_AND_INSTANCE_MESSAGE = (
        "CPU and instance configured together, Invalid configuration."
    )
    VALIDATION_FAILURE_CPU_AND_INSTANCE_SUGGESTION = (
        "Please configure only CPU or instance"
    )
    VALIDATION_FAILURE_GPU_AND_INSTANCE_MESSAGE = (
        "GPU and instance configured together, Invalid configuration."
    )
    VALIDATION_FAILURE_GPU_AND_INSTANCE_SUGGESTION = (
        "Please configure only GPU or instance"
    )
    EXISTING_MODEL_PASSED_MSG = "Build process was provided with pre built model"
    INVALID_QWAK_MODEL_MSG_FORMAT = (
        "The provided qwak model is not of {expected_class} but of {passed_class}"
    )
    SERVICE_ACCOUNT_NOT_FOUND = (
        "GCP service account key secret '{secret_name}' wasn't found"
    )
    SERVICE_ACCOUNT_NOT_FOUND_SUGGESTION = "Make sure your GCP service account key secret is configured correctly in the environment"

    def description(self) -> str:
        return self.STEP_DESCRIPTION

    def execute(self) -> None:
        self.validate_build_properties()
        self.create_build_id()
        self.collect_model_id()
        self.create_build_dir()
        self.collect_git_credentials()
        self.validate_dependencies()
        self.validate_resources()
        self.validate_deployment_resources()
        self.validate_qwak_model()
        self.validate_gcp_service_account_secret()

    def validate_build_properties(self):
        if not self.config.build_properties.model_uri.uri:
            error_message = "Model uri wasn't set"
            self.build_logger.error(f"{error_message}, failing...")

            raise QwakSuggestionException(
                message=error_message,
                suggestion="Make sure your build properties object contains model uri argument",
            )
        if (
            self.config.build_properties.model_uri.git_credentials_secret
            and self.config.build_properties.model_uri.git_secret_ssh
        ):
            error_message = (
                "Only one of git credentials secret or git secret ssh can be configured"
            )
            raise QwakSuggestionException(
                message=error_message,
                suggestion="Make sure your git credentials or git secret ssh are configured correctly and only one of "
                "them is configured",
            )

    def collect_model_id(self):
        model_id = self.config.build_properties.model_id
        model = self.context.client_models_management.get_model(
            model_id=model_id,
            exception_on_missing=False,
        )
        if not model:
            suggestion = f"Create model {model_id} or check model ID spelling"
            snake_case_model_id = snake_case(model_id)
            if self.context.client_models_management.is_model_exists(
                snake_case_model_id
            ):
                suggestion = f"Try using model ID {snake_case_model_id} instead"
            raise QwakSuggestionException(
                message=f"Model ID {model_id} isn't found",
                suggestion=suggestion,
            )
        self.context.project_uuid = model.project_id
        self.context.model_uuid = model.uuid
        self.context.model_id = model_id

    def create_build_id(self):
        if not self.context.build_id:
            self.context.build_id = str(uuid.uuid4())
            self.build_logger.info(f"Generated build ID - {self.context.build_id}")
        else:
            self.build_logger.info(f"Using given build ID - {self.context.build_id}")

    def create_build_dir(self):
        build_dir = HOST_TEMP_BUILD_DIR / self.context.model_id / self.context.build_id
        build_dir.mkdir(exist_ok=True, parents=True)
        self.context.host_temp_local_build_dir = build_dir
        self.build_logger.debug(f"Build directory created - {build_dir}")

    def collect_git_credentials(self):
        if self.config.build_properties.model_uri.git_credentials:
            self.context.git_credentials = (
                self.config.build_properties.model_uri.git_credentials
            )
        elif self.config.build_properties.model_uri.git_credentials_secret:
            self.context.git_credentials = SecretServiceClient().get_secret(
                self.config.build_properties.model_uri.git_credentials_secret
            )
        elif self.config.build_properties.model_uri.git_secret_ssh:
            self.context.git_ssh_key = SecretServiceClient().get_secret(
                self.config.build_properties.model_uri.git_secret_ssh
            )
        else:
            self.build_logger.debug("Git credentials isn't configured")

    def validate_dependencies(self):
        if (
            Path(self.config.build_properties.model_uri.uri).is_dir()
            and not self.config.build_env.python_env.dependency_file_path
        ):
            model_uri, main_dir = (
                Path(self.config.build_properties.model_uri.uri),
                self.config.build_properties.model_uri.main_dir,
            )
            (
                self.context.dependency_manager_type,
                self.context.model_relative_dependency_file,
                self.context.model_relative_dependency_lock_file,
            ) = find_dependency_files(model_uri, main_dir, self.build_logger)

            if (
                self.context.dependency_manager_type
                and self.context.model_relative_dependency_file
            ):
                return

            self.build_logger.error("Dependency file wasn't found, failing...")
            raise QwakSuggestionException(
                message="Dependency file isn't found",
                suggestion="Make sure your model include one of dependencies manager: pip/poetry/conda",
            )

    def validate_resources(self):
        gpu_configured = (
            self.config.build_env.remote.resources.gpu_type
            or self.config.build_env.remote.resources.gpu_amount
        )
        cpu_configured = (
            self.config.build_env.remote.resources.cpus
            or self.config.build_env.remote.resources.memory
        )
        instance_configured = self.config.build_env.remote.resources.instance

        if cpu_configured and gpu_configured:
            raise QwakSuggestionException(
                message=self.VALIDATION_FAILURE_CPU_AND_GPU_MESSAGE,
                suggestion=self.VALIDATION_FAILURE_CPU_AND_GPU_SUGGESTION,
            )
        if cpu_configured and instance_configured:
            raise QwakSuggestionException(
                message=self.VALIDATION_FAILURE_CPU_AND_INSTANCE_MESSAGE,
                suggestion=self.VALIDATION_FAILURE_CPU_AND_INSTANCE_SUGGESTION,
            )
        if gpu_configured and instance_configured:
            raise QwakSuggestionException(
                message=self.VALIDATION_FAILURE_GPU_AND_INSTANCE_MESSAGE,
                suggestion=self.VALIDATION_FAILURE_GPU_AND_INSTANCE_SUGGESTION,
            )

        if instance_configured:
            provider = self.__get_provider()
            verify_template_id(
                self.config.build_env.remote.resources.instance,
                self.context.client_instance_template,
                provider=provider,
            )

        if not (cpu_configured or gpu_configured or instance_configured):
            self.config.build_env.remote.resources.cpus = self.DEFAULT_CPUS
            self.config.build_env.remote.resources.memory = self.DEFAULT_MEMORY

    def __get_provider(self):
        user_context = self.context.client_ecosystem.get_authenticated_user_context()
        provider = None
        if (
            user_context.user.environment_details.configuration.cloud_configuration.WhichOneof(
                "configuration"
            )
            == "aws_cloud_configuration"
        ):
            provider = Provider.AWS
        elif (
            user_context.user.environment_details.configuration.cloud_configuration.WhichOneof(
                "configuration"
            )
            == "gcp_cloud_configuration"
        ):
            provider = Provider.GCP
        return provider

    def validate_deployment_resources(self):
        if self.config.deploy and self.config.deployment_instance:
            provider = self.__get_provider()
            verify_template_id(
                self.config.deployment_instance,
                self.context.client_instance_template,
                provider=provider,
            )

    def validate_qwak_model(self):
        if self.config.pre_built_model:
            self.build_logger.debug(self.EXISTING_MODEL_PASSED_MSG)
            if not isinstance(self.config.pre_built_model, QwakModel):
                raise QwakException(
                    self.INVALID_QWAK_MODEL_MSG_FORMAT.format(
                        expected_class=QwakModel.__name__,
                        passed_class=self.config.pre_built_model.__class__,
                    )
                )

    def validate_gcp_service_account_secret(self):
        if self.config.build_env.docker.service_account_key_secret_name:
            self.build_logger.debug("Checking GCP service account key secret")
            try:
                SecretServiceClient().get_secret(
                    self.config.build_env.docker.service_account_key_secret_name
                )
            except QwakException as e:
                if e.status_code == grpc.StatusCode.NOT_FOUND:
                    raise QwakSuggestionException(
                        message=self.SERVICE_ACCOUNT_NOT_FOUND.format(
                            secret_name=self.config.build_env.docker.service_account_key_secret_name
                        ),
                        suggestion=self.SERVICE_ACCOUNT_NOT_FOUND_SUGGESTION,
                    )
                raise e

            self.build_logger.debug("service account key secret found")



import uuid

import grpc
from _qwak_proto.qwak.feature_store.features.feature_set_pb2 import (
    DeployedModelInUseLink,
    FeatureSet,
    FeatureSetDefinition,
    FeatureSetMetadata,
    FeaturesetSchedulingState,
    FeatureStatus,
)
from _qwak_proto.qwak.feature_store.features.feature_set_service_pb2 import (
    DeleteFeatureSetResponse,
    GetFeatureSetByNameResponse,
    GetFeaturesetSchedulingStateResponse,
    ListFeatureSetsResponse,
    RegisterFeatureSetResponse,
    UpdateFeatureSetResponse,
)
from _qwak_proto.qwak.feature_store.features.feature_set_service_pb2_grpc import (
    FeatureSetServiceServicer,
)


class FeatureSetServiceMock(FeatureSetServiceServicer):
    def __init__(self):
        self._features_spec = {}

    def reset_features(self):
        self._features_spec.clear()

    def RegisterFeatureSet(self, request, context):
        fs_id = str(uuid.uuid4())
        self._features_spec[fs_id] = request.feature_set_spec
        return RegisterFeatureSetResponse(
            feature_set=FeatureSet(
                feature_set_definition=FeatureSetDefinition(
                    feature_set_id=fs_id, feature_set_spec=request.feature_set_spec
                )
            )
        )

    def UpdateFeatureSet(self, request, context):
        fs_id: str = request.feature_set_id
        self._features_spec[fs_id] = request.feature_set_spec
        return UpdateFeatureSetResponse(
            feature_set=FeatureSet(
                feature_set_definition=FeatureSetDefinition(
                    feature_set_id=fs_id, feature_set_spec=request.feature_set_spec
                )
            )
        )

    def DeleteFeatureSet(self, request, context):
        if request.feature_set_id in self._features_spec:
            self._features_spec.pop(request.feature_set_id)
            return DeleteFeatureSetResponse()

        context.set_details(f"Feature set ID {request.feature_set_id} doesn't exist'")
        context.set_code(grpc.StatusCode.NOT_FOUND)

    def GetFeatureSetByName(self, request, context):
        feature_sets = [
            (fs_id, fs_spec)
            for (fs_id, fs_spec) in self._features_spec.items()
            if fs_spec.name == request.feature_set_name
        ]
        if feature_sets:
            fs_id, fs_spec = feature_sets[0]
            return GetFeatureSetByNameResponse(
                feature_set=FeatureSet(
                    feature_set_definition=FeatureSetDefinition(
                        feature_set_id=fs_id,
                        feature_set_spec=fs_spec,
                        status=FeatureStatus.VALID,
                    ),
                    metadata=FeatureSetMetadata(),
                    deployed_models_in_use_link=[DeployedModelInUseLink()],
                )
            )

        context.set_details(
            f"Feature set named {request.feature_set_name} doesn't exist'"
        )
        context.set_code(grpc.StatusCode.NOT_FOUND)

    def ListFeatureSets(self, request, context):
        return ListFeatureSetsResponse(
            feature_families=[
                FeatureSet(
                    feature_set_definition=FeatureSetDefinition(
                        feature_set_spec=feature_spec, status=FeatureStatus.VALID
                    ),
                    metadata=FeatureSetMetadata(),
                    deployed_models_in_use_link=[DeployedModelInUseLink()],
                )
                for feature_spec in self._features_spec.values()
            ]
        )

    def GetFeaturesetSchedulingState(self, request, context):
        return GetFeaturesetSchedulingStateResponse(
            state=FeaturesetSchedulingState.SCHEDULING_STATE_ENABLED
        )



import json
import re
from typing import List, Optional

import yaml
from _qwak_proto.qwak.builds.build_pb2 import (
    BuildEnv,
    BuildProperties,
    BuildPropertiesV1,
    DockerEnv,
    MemoryUnit,
    ModelUriSpec,
    PythonEnv,
    RemoteBuildSpec,
)
from _qwak_proto.qwak.fitness_service.fitness_pb2 import PurchaseOption
from _qwak_proto.qwak.user_application.common.v0.resources_pb2 import (
    CpuResources,
    GpuResources,
    PodComputeResourceTemplateSpec,
)
from qwak.exceptions import QwakException
from yaml import Loader


def _get_build_model_spec(
    build_conf,
    verbose: int = 3,
    git_commit_id: str = "",
    resolved_model_url: str = "",
    build_code_path: str = "",
    build_v1_flag: bool = False,
    build_config_url: str = "",
    qwak_sdk_wheel_url: str = "",
    qwak_sdk_version_url: str = "",
    build_steps: Optional[List[str]] = None,
    sdk_version: str = "",
) -> RemoteBuildSpec:
    build_spec = RemoteBuildSpec(
        build_properties=BuildProperties(
            build_id=build_conf.build_properties.build_id,
            model_id=build_conf.build_properties.model_id,
            branch=build_conf.build_properties.branch,
            tags=build_conf.build_properties.tags,
            gpu_compatible=build_conf.build_properties.gpu_compatible,
            model_uri=ModelUriSpec(
                uri=resolved_model_url or build_conf.build_properties.model_uri.uri,
                git_credentials=build_conf.build_properties.model_uri.git_credentials,
                git_credentials_secret=build_conf.build_properties.model_uri.git_credentials_secret,
                git_branch=build_conf.build_properties.model_uri.git_branch,
                commit_id=git_commit_id,
                main_dir=build_conf.build_properties.model_uri.main_dir,
            ),
        ),
        build_env=BuildEnv(
            docker_env=DockerEnv(
                base_image=build_conf.build_env.docker.base_image,
                assumed_iam_role_arn=build_conf.build_env.docker.assumed_iam_role_arn,
                service_account_key_secret_name=build_conf.build_env.docker.service_account_key_secret_name,
                no_cache=not build_conf.build_env.docker.cache,
                env_vars=build_conf.build_env.docker.env_vars,
            ),
            python_env=PythonEnv(
                git_credentials=build_conf.build_properties.model_uri.git_credentials,
                git_credentials_secret=build_conf.build_properties.model_uri.git_credentials_secret,
                qwak_sdk_version=sdk_version,
            ),
        ),
        verbose=verbose,
        build_code_path=build_code_path,
        build_config=json.dumps(
            yaml.load(build_conf.to_yaml(), Loader=Loader)  # nosec B506
        ),
        build_properties_v1=BuildPropertiesV1(
            build_config_url=build_config_url,
            qwak_sdk_wheel_url=qwak_sdk_wheel_url,
            qwak_sdk_version_url=qwak_sdk_version_url,
        ),
        build_v1_flag=build_v1_flag,
        build_steps=build_steps,
        purchase_option=_purchase_option_to_enum(build_conf.purchase_option),
        provision_instance_timeout=build_conf.provision_instance_timeout,
    )

    if build_conf.build_env.remote.resources.instance:
        build_spec.client_pod_compute_resources.template_spec.CopyFrom(
            PodComputeResourceTemplateSpec(
                template_id=build_conf.build_env.remote.resources.instance
            )
        )
    elif build_conf.build_env.remote.resources.gpu_type:
        build_spec.client_pod_compute_resources.gpu_resources.CopyFrom(
            GpuResources(
                gpu_type=build_conf.build_env.remote.resources.gpu_type,
                gpu_amount=build_conf.build_env.remote.resources.gpu_amount,
            )
        )
    else:
        build_spec.client_pod_compute_resources.cpu_resources.CopyFrom(
            CpuResources(
                cpu=build_conf.build_env.remote.resources.cpus,
                memory_amount=int(
                    re.sub(
                        r"\D",
                        "",
                        build_conf.build_env.remote.resources.memory,
                    )
                ),
                memory_units=map_memory_units(
                    build_conf.build_env.remote.resources.memory
                ),
            )
        )

    return build_spec


def map_memory_units(memory):
    memory_unit = re.sub(r"\d+", "", memory)
    if memory_unit == "Gi":
        return MemoryUnit.GIB
    elif memory_unit == "Mib":
        return MemoryUnit.MIB
    else:
        return MemoryUnit.UNKNOWN_MEMORY_UNIT


def _purchase_option_to_enum(purchase_option: Optional[str]) -> PurchaseOption:
    if purchase_option == "" or purchase_option is None:
        return PurchaseOption.INVALID_PURCHASE_OPTION
    elif purchase_option == "ondemand":
        return PurchaseOption.ONDEMAND_PURCHASE_OPTION
    elif purchase_option == "spot":
        return PurchaseOption.SPOT_PURCHASE_OPTION
    else:
        raise QwakException("Purchase option must be either 'ondemand' or 'spot'")



from typing import List


def _generate_new_col(col_name: str, duplicate_cols_dict: dict):
    """
    Generate column without leading feature set name.
    Args:
        col_name: the target column name
        duplicate_cols_dict: duplicate features dictionary
    Returns list of the new columns
    """
    feature_full_name: List[str] = col_name.split(".")
    if len(feature_full_name) != 2:
        return col_name
    elif len(feature_full_name) == 2 and duplicate_cols_dict[feature_full_name[1]] > 1:
        return col_name
    elif len(feature_full_name) == 2 and duplicate_cols_dict[feature_full_name[1]] == 1:
        return feature_full_name[1]


def normalize_cols(cols) -> List[str]:
    """
    Normalize cols - try to remove leading feature set name from features
    Args:
        cols: list of column
    Return normalized columns
    """
    duplicate_columns_dict = {}
    for col in cols:
        feature_full_name: List[str] = col.split(".")
        if len(feature_full_name) != 2:
            duplicate_columns_dict[col] = 1
        else:
            fs_name = feature_full_name[1]
            duplicate_columns_dict[fs_name] = duplicate_columns_dict.get(fs_name, 0) + 1
    cols = [_generate_new_col(col_name, duplicate_columns_dict) for col_name in cols]
    return cols



import grpc
from _qwak_proto.qwak.administration.v0.environments.environment_pb2 import (
    QwakEnvironmentStatus,
)
from _qwak_proto.qwak.administration.v0.environments.environment_service_pb2 import (
    GetEnvironmentApplicationUserCredentialsRequest,
    GetEnvironmentApplicationUserCredentialsResponse,
    ListEnvironmentsRequest,
    ListEnvironmentsResponse,
)
from _qwak_proto.qwak.administration.v0.environments.environment_service_pb2_grpc import (
    EnvironmentServiceStub,
)
from dependency_injector.wiring import Provide
from qwak.exceptions import QwakException
from qwak.inner.di_configuration import QwakContainer


class EnvironmentClient:
    """
    Used for interacting with Qwak's Environemt
    """

    def __init__(self, grpc_channel=Provide[QwakContainer.core_grpc_channel]):
        self._environment_service = EnvironmentServiceStub(grpc_channel)

    def list_environments(self) -> ListEnvironmentsResponse:
        """
        List of all environment without filter
        Return ListEnvironmentsResponse
        """
        request = ListEnvironmentsRequest()
        try:
            return self._environment_service.ListEnvironments(request)
        except grpc.RpcError as e:
            raise QwakException(
                f"Failed to get List of environments, error is {e.details()}"
            )

    def list_environments_by_status(
        self, status: QwakEnvironmentStatus
    ) -> ListEnvironmentsResponse:
        """
        List of all environment without filter
        Args: status which filter environment by
        Return: ListEnvironmentsResponse

        """
        try:
            request = ListEnvironmentsRequest(
                filter=ListEnvironmentsRequest.Filter(environment_status=status)
            )
            return self._environment_service.ListEnvironments(request)
        except grpc.RpcError as e:
            raise QwakException(
                f"Failed to get List of environments with status, error is {e.details()}"
            )

    def get_environment_application_user(
        self, environment_id: str
    ) -> GetEnvironmentApplicationUserCredentialsResponse:
        """
        Get application user by environment id
        Return: GetEnvironmentApplicationUserCredentialsResponse
        """
        request = GetEnvironmentApplicationUserCredentialsRequest(
            environment_id=environment_id
        )
        try:
            return self._environment_service.GetEnvironmentApplicationUserCredentials(
                request
            )
        except grpc.RpcError as e:
            raise QwakException(
                f"Failed to get application user, error is {e.details()}"
            )



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import qwak.execution.v1.state.spark_execution_state_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class ExecutionState(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    EXECUTION_ID_FIELD_NUMBER: builtins.int
    FEATURESET_ID_FIELD_NUMBER: builtins.int
    ENVIRONMENT_ID_FIELD_NUMBER: builtins.int
    SPARK_APP_STATE_FIELD_NUMBER: builtins.int
    execution_id: builtins.str
    """Execution Id"""
    featureset_id: builtins.str
    """featureset id this execution belongs to (featureset:execution is 1:many relation)"""
    environment_id: builtins.str
    """environmentId of the featureset"""
    @property
    def spark_app_state(self) -> qwak.execution.v1.state.spark_execution_state_pb2.SparkExecutionState: ...
    def __init__(
        self,
        *,
        execution_id: builtins.str = ...,
        featureset_id: builtins.str = ...,
        environment_id: builtins.str = ...,
        spark_app_state: qwak.execution.v1.state.spark_execution_state_pb2.SparkExecutionState | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["exec_state", b"exec_state", "spark_app_state", b"spark_app_state"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["environment_id", b"environment_id", "exec_state", b"exec_state", "execution_id", b"execution_id", "featureset_id", b"featureset_id", "spark_app_state", b"spark_app_state"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["exec_state", b"exec_state"]) -> typing_extensions.Literal["spark_app_state"] | None: ...

global___ExecutionState = ExecutionState



from typing import Optional

from _qwak_proto.qwak.feature_store.features.feature_set_pb2 import FeatureSetSpec
from _qwak_proto.qwak.feature_store.sources.data_source_pb2 import DataSourceSpec
from _qwak_proto.qwak.features_operator.v3.features_operator_async_service_pb2 import (
    DataSourceValidationOptions,
    FeatureSetValidationOptions,
    GetValidationResultRequest,
    GetValidationResultResponse,
    ValidateDataSourceRequest,
    ValidateFeatureSetRequest,
    ValidationResponse,
)
from _qwak_proto.qwak.features_operator.v3.features_operator_async_service_pb2_grpc import (
    FeaturesOperatorAsyncServiceStub,
)
from _qwak_proto.qwak.features_operator.v3.features_operator_pb2 import (
    ValidationNotReadyResponse,
)
from dependency_injector.wiring import Provide
from qwak.inner.di_configuration import QwakContainer
from qwak.inner.tool.retry_utils import retry


class ValidationNotReadyException(Exception):
    pass


class ValidationTimeoutException(Exception):
    pass


class FeaturesOperatorClient:
    """
    Validates and samples features store objects like: data sources and feature sets.
    """

    def __init__(self, grpc_channel=Provide[QwakContainer.core_grpc_channel]):
        self._v3_client = FeaturesOperatorAsyncServiceStub(grpc_channel)

    def validate_data_source(
        self,
        data_source_spec: DataSourceSpec,
        num_samples: int = 10,
        validation_options: Optional[DataSourceValidationOptions] = None,
    ) -> str:
        """
        Validates and fetches a sample from the data source
        :return: Request handle id to poll for result with
        """
        request: ValidateDataSourceRequest = ValidateDataSourceRequest(
            data_source_spec=data_source_spec,
            num_samples=num_samples,
            validation_options=validation_options,
        )
        response: ValidationResponse = self._v3_client.ValidateDataSource(request)
        return response.request_id

    def validate_featureset(
        self,
        featureset_spec: FeatureSetSpec,
        resource_path: Optional[str] = None,
        num_samples: int = 10,
        validation_options: Optional[FeatureSetValidationOptions] = None,
    ) -> str:
        """
        Validates and fetches a sample from the featureset
        :return: Request handle id to poll for result with
        """
        resource_path = str(resource_path) if resource_path is not None else None
        request: ValidateFeatureSetRequest = ValidateFeatureSetRequest(
            feature_set_spec=featureset_spec,
            num_samples=num_samples,
            zip_path=resource_path,
            validation_options=validation_options,
        )

        response: ValidationResponse = self._v3_client.ValidateFeatureSet(request)
        return response.request_id

    def get_result(self, request_handle: str) -> GetValidationResultResponse:
        request: GetValidationResultRequest = GetValidationResultRequest(
            request_id=request_handle
        )
        response: GetValidationResultResponse = self._v3_client.GetValidationResult(
            request
        )

        return response

    def _inner_poll(self, request_handle: str) -> GetValidationResultResponse:
        response: GetValidationResultResponse = self.get_result(
            request_handle=request_handle
        )
        response_type = getattr(response, response.WhichOneof("type"))

        if isinstance(response_type, ValidationNotReadyResponse):
            raise ValidationNotReadyException()

        return response

    def poll_for_result(
        self,
        request_handle: str,
        timeout_seconds: int = 5 * 60,
        poll_interval_seconds: int = 3,
    ) -> GetValidationResultResponse:
        try:
            result = retry(
                f=self._inner_poll,
                kwargs={"request_handle": request_handle},
                exceptions=ValidationNotReadyException,
                attempts=timeout_seconds / poll_interval_seconds,
                delay=poll_interval_seconds,
            )
        except ValidationNotReadyException:
            raise ValidationTimeoutException(
                f"Validation timed out. Qwak limits validation execution time to {timeout_seconds} seconds"
            )

        return result

    def validate_data_source_blocking(
        self,
        data_source_spec: DataSourceSpec,
        num_samples: int = 10,
        timeout_seconds: int = 5 * 60,
        poll_interval_seconds: int = 3,
        validation_options: Optional[DataSourceValidationOptions] = None,
    ) -> GetValidationResultResponse:
        request_handle: str = self.validate_data_source(
            data_source_spec=data_source_spec,
            num_samples=num_samples,
            validation_options=validation_options,
        )

        return self.poll_for_result(
            request_handle=request_handle,
            timeout_seconds=timeout_seconds,
            poll_interval_seconds=poll_interval_seconds,
        )

    def validate_featureset_blocking(
        self,
        featureset_spec: FeatureSetSpec,
        resource_path: Optional[str] = None,
        num_samples: int = 10,
        timeout_seconds: int = 5 * 60,
        poll_interval_seconds: int = 3,
        validation_options: Optional[FeatureSetValidationOptions] = None,
    ) -> GetValidationResultResponse:
        request_handle: str = self.validate_featureset(
            featureset_spec=featureset_spec,
            resource_path=resource_path,
            num_samples=num_samples,
            validation_options=validation_options,
        )

        return self.poll_for_result(
            request_handle=request_handle,
            timeout_seconds=timeout_seconds,
            poll_interval_seconds=poll_interval_seconds,
        )
